@@ -67,6 +67,7 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 	g_config->getOption("SDL.ShowFrameCount", &frame_display);
 	g_config->getOption("SDL.ShowLagCount", &lagCounterDisplay);
 	g_config->getOption("SDL.ShowRerecordCount", &rerecord_display);
+	g_config->getOption("SDL.ShowGuiMessages", &vidGuiMsgEna);
 	
 	style = this->style();
 
@@ -84,8 +85,9 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 
 	driverSelect = new QComboBox();
 
-	driverSelect->addItem( tr("OpenGL"), 0 );
-	driverSelect->addItem( tr("SDL"), 1 );
+	driverSelect->addItem( tr("OpenGL"), ConsoleViewerBase::VIDEO_DRIVER_OPENGL );
+	driverSelect->addItem( tr("SDL"), ConsoleViewerBase::VIDEO_DRIVER_SDL );
+	driverSelect->addItem( tr("QPainter"), ConsoleViewerBase::VIDEO_DRIVER_QPAINTER );
 	
 	hbox1 = new QHBoxLayout();
 
@@ -171,6 +173,10 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 	// Clip Sides Checkbox
 	clipSidesCbx  = new QCheckBox( tr("Clip Left/Right Sides (8 px on each)") );
 
+	// Show GUI Messages Checkbox
+	showGuiMsgs_cbx  = new QCheckBox( tr("GUI Messages") );
+	showGuiMsgs_cbx->setChecked( vidGuiMsgEna );
+
 	// Show FPS Checkbox
 	showFPS_cbx  = new QCheckBox( tr("Frames Per Second") );
 
@@ -227,15 +233,10 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 	
 	if ( consoleWindow )
 	{
-		if ( consoleWindow->viewport_GL )
+		if ( consoleWindow->viewport_Interface )
 		{
-			autoScaleCbx->setChecked( consoleWindow->viewport_GL->getAutoScaleOpt() );
-			aspectCbx->setChecked( consoleWindow->viewport_GL->getForceAspectOpt() );
-		}
-		else if ( consoleWindow->viewport_SDL )
-		{
-			autoScaleCbx->setChecked( consoleWindow->viewport_SDL->getAutoScaleOpt() );
-			aspectCbx->setChecked( consoleWindow->viewport_SDL->getForceAspectOpt() );
+			autoScaleCbx->setChecked( consoleWindow->viewport_Interface->getAutoScaleOpt() );
+			aspectCbx->setChecked( consoleWindow->viewport_Interface->getForceAspectOpt() );
 		}
 	}
 
@@ -247,6 +248,7 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 	connect(sprtLimCbx      , SIGNAL(stateChanged(int)), this, SLOT(useSpriteLimitChanged(int)) );
 	connect(clipSidesCbx    , SIGNAL(stateChanged(int)), this, SLOT(clipSidesChanged(int)) );
 	connect(showFPS_cbx     , SIGNAL(stateChanged(int)), this, SLOT(showFPSChanged(int)) );
+	connect(showGuiMsgs_cbx , SIGNAL(stateChanged(int)), this, SLOT(showGuiMsgsChanged(int)) );
 	connect(aspectCbx       , SIGNAL(stateChanged(int)), this, SLOT(aspectEnableChanged(int)) );
 	connect(autoScaleCbx    , SIGNAL(stateChanged(int)), this, SLOT(autoScaleChanged(int)) );
 	connect(drawInputAidsCbx, SIGNAL(stateChanged(int)), this, SLOT(drawInputAidsChanged(int)) );
@@ -299,15 +301,10 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 
 	if ( consoleWindow )
 	{
-		if ( consoleWindow->viewport_GL )
+		if ( consoleWindow->viewport_Interface )
 		{
-			xScaleBox->setValue( consoleWindow->viewport_GL->getScaleX() );
-			yScaleBox->setValue( consoleWindow->viewport_GL->getScaleY() );
-		}
-		else if ( consoleWindow->viewport_SDL )
-		{
-			xScaleBox->setValue( consoleWindow->viewport_SDL->getScaleX() );
-			yScaleBox->setValue( consoleWindow->viewport_SDL->getScaleY() );
+			xScaleBox->setValue( consoleWindow->viewport_Interface->getScaleX() );
+			yScaleBox->setValue( consoleWindow->viewport_Interface->getScaleY() );
 		}
 	}
 
@@ -366,6 +363,7 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 	vbox3->addStretch(5);
 	 gbox->setLayout( vbox );
 
+	vbox->addWidget( showGuiMsgs_cbx );
 	vbox->addWidget( drawInputAidsCbx );
 	vbox->addWidget( showFPS_cbx );
 	vbox->addWidget( showFrameCount_cbx );
@@ -527,14 +525,14 @@ ConsoleVideoConfDialog_t::ConsoleVideoConfDialog_t(QWidget *parent)
 //----------------------------------------------------
 ConsoleVideoConfDialog_t::~ConsoleVideoConfDialog_t(void)
 {
-	printf("Destroy Video Config Window\n");
+	//printf("Destroy Video Config Window\n");
 
 	updateTimer->stop();
 }
 //----------------------------------------------------------------------------
 void ConsoleVideoConfDialog_t::closeEvent(QCloseEvent *event)
 {
-	printf("Video Config Close Window Event\n");
+	//printf("Video Config Close Window Event\n");
 	done(0);
 	deleteLater();
 	event->accept();
@@ -587,13 +585,9 @@ void ConsoleVideoConfDialog_t::updateReadouts(void)
 
 		w = consoleWindow->size();
 
-		if ( consoleWindow->viewport_GL )
+		if ( consoleWindow->viewport_Interface )
 		{
-			v = consoleWindow->viewport_GL->size();
-		}
-		else if ( consoleWindow->viewport_SDL )
-		{
-			v = consoleWindow->viewport_SDL->size();
+			v = consoleWindow->viewport_Interface->size();
 		}
 
 		sprintf( stmp, "%i x %i ", w.width(), w.height() );
@@ -719,13 +713,9 @@ void ConsoleVideoConfDialog_t::openGL_linearFilterChanged( int value )
 
    if ( consoleWindow != NULL )
    {
-      if ( consoleWindow->viewport_GL )
+      if ( consoleWindow->viewport_Interface )
       {
-         consoleWindow->viewport_GL->setLinearFilterEnable( opt );
-      }
-      if ( consoleWindow->viewport_SDL )
-      {
-         consoleWindow->viewport_SDL->setLinearFilterEnable( opt );
+         consoleWindow->viewport_Interface->setLinearFilterEnable( opt );
       }
    }
 }
@@ -738,13 +728,9 @@ void ConsoleVideoConfDialog_t::autoScaleChanged( int value )
 
    if ( consoleWindow != NULL )
    {
-      if ( consoleWindow->viewport_GL )
+      if ( consoleWindow->viewport_Interface )
       {
-         consoleWindow->viewport_GL->setAutoScaleOpt( opt );
-      }
-      if ( consoleWindow->viewport_SDL )
-      {
-         consoleWindow->viewport_SDL->setAutoScaleOpt( opt );
+         consoleWindow->viewport_Interface->setAutoScaleOpt( opt );
       }
    }
 }
@@ -825,9 +811,9 @@ void ConsoleVideoConfDialog_t::vsync_changed( int value )
 			//consoleWindow->viewport_GL->setVsyncEnable( opt );
 			consoleWindow->loadVideoDriver( 0, true );
 		}
-		if ( consoleWindow->viewport_SDL )
+		else if ( consoleWindow->viewport_Interface )
 		{
-			consoleWindow->viewport_SDL->setVsyncEnable( opt );
+			consoleWindow->viewport_Interface->setVsyncEnable( opt );
 		}
 	}
 }
@@ -879,6 +865,14 @@ void ConsoleVideoConfDialog_t::showFPSChanged( int value )
 	FCEU_WRAPPER_UNLOCK();
 }
 //----------------------------------------------------
+void ConsoleVideoConfDialog_t::showGuiMsgsChanged( int value )
+{
+	//printf("Value:%i \n", value );
+	vidGuiMsgEna = (value == Qt::Checked);
+	g_config->setOption("SDL.ShowGuiMessages", vidGuiMsgEna );
+	g_config->save ();
+}
+//----------------------------------------------------
 void ConsoleVideoConfDialog_t::aspectEnableChanged( int value )
 {
 	//printf("Value:%i \n", value );
@@ -1079,15 +1073,10 @@ QSize ConsoleVideoConfDialog_t::calcNewScreenSize(void)
 
 		w = consoleWindow->size();
 
-		if ( consoleWindow->viewport_GL )
+		if ( consoleWindow->viewport_Interface )
 		{
-			v = consoleWindow->viewport_GL->size();
-			aspectRatio = consoleWindow->viewport_GL->getAspectRatio();
-		}
-		else if ( consoleWindow->viewport_SDL )
-		{
-			v = consoleWindow->viewport_SDL->size();
-			aspectRatio = consoleWindow->viewport_SDL->getAspectRatio();
+			v = consoleWindow->viewport_Interface->size();
+			aspectRatio = consoleWindow->viewport_Interface->getAspectRatio();
 		}
 
 		dw = w.width()  - v.width();
@@ -1152,21 +1141,13 @@ void ConsoleVideoConfDialog_t::applyChanges( void )
 		g_config->setOption("SDL.WinSizeX", s.width() );
 		g_config->setOption("SDL.WinSizeY", s.height() );
 
-		if ( consoleWindow->viewport_GL )
+		if ( consoleWindow->viewport_Interface )
 		{
-         		consoleWindow->viewport_GL->setLinearFilterEnable( gl_LF_chkBox->isChecked() );
-			consoleWindow->viewport_GL->setForceAspectOpt( aspectCbx->isChecked() );
-			consoleWindow->viewport_GL->setAutoScaleOpt( autoScaleCbx->isChecked() );
-			consoleWindow->viewport_GL->setScaleXY( xscale, yscale );
-			consoleWindow->viewport_GL->reset();
-		}
-		if ( consoleWindow->viewport_SDL )
-		{
-         		consoleWindow->viewport_SDL->setLinearFilterEnable( gl_LF_chkBox->isChecked() );
-			consoleWindow->viewport_SDL->setForceAspectOpt( aspectCbx->isChecked() );
-			consoleWindow->viewport_SDL->setAutoScaleOpt( autoScaleCbx->isChecked() );
-			consoleWindow->viewport_SDL->setScaleXY( xscale, yscale );
-			consoleWindow->viewport_SDL->reset();
+         		consoleWindow->viewport_Interface->setLinearFilterEnable( gl_LF_chkBox->isChecked() );
+			consoleWindow->viewport_Interface->setForceAspectOpt( aspectCbx->isChecked() );
+			consoleWindow->viewport_Interface->setAutoScaleOpt( autoScaleCbx->isChecked() );
+			consoleWindow->viewport_Interface->setScaleXY( xscale, yscale );
+			consoleWindow->viewport_Interface->reset();
 		}
 
 		if ( !consoleWindow->isFullScreen() && !consoleWindow->isMaximized() )
