@@ -481,7 +481,7 @@ void HighlightSyntax(HWND hWnd, int lines)
 	{
 		commentline = false;
 		wordbreak = SendDlgItemMessage(hWnd, IDC_DEBUGGER_DISASSEMBLY, EM_FINDWORDBREAK, (WPARAM)WB_RIGHT, (LPARAM)newline.chrg.cpMin + 21);
-		for (int ch = newline.chrg.cpMin; ; ch++)
+		for (int ch = newline.chrg.cpMin; debug_wstr[ch] != 0; ch++)
 		{
 			if (debug_wstr[ch] == L'=' || debug_wstr[ch] == L'@' || debug_wstr[ch] == L'\n' || debug_wstr[ch] == L'-' || debug_wstr[ch] == L';')
 			{
@@ -1235,7 +1235,7 @@ void DeleteBreak(int sel)
 	if(sel<0) return;
 	if(sel>=numWPs) return;
 	if (watchpoint[sel].cond)
-		freeTree(watchpoint[sel].cond);
+		delete watchpoint[sel].cond;
 	if (watchpoint[sel].condText)
 		free(watchpoint[sel].condText);
 	if (watchpoint[sel].desc)
@@ -1471,7 +1471,7 @@ INT_PTR CALLBACK PatcherCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 							else
 								iapoffset = GetNesFileAddress(GetEditHex(hwndDlg,IDC_ROMPATCHER_OFFSET));
 							if((iapoffset < 16) && (iapoffset != -1)){
-								MessageBox(hDebug, "Sorry, iNES Header editing isn't supported by this tool. If you want to edit the header, please use iNES Header Editor", "Error", MB_OK | MB_ICONASTERISK);
+								MessageBox(hDebug, "Sorry, NES Header editing isn't supported by this tool. If you want to edit the header, please use NES Header Editor", "Error", MB_OK | MB_ICONASTERISK);
 								iapoffset = -1;
 							}
 							if((iapoffset > PRGsize[0]) && (iapoffset != -1)){
@@ -1696,35 +1696,47 @@ int Debugger_CheckClickingOnAnAddressOrSymbolicName(unsigned int lineNumber, boo
 		if (sel_end > sel_start)
 			return EOF;
 
-	// find the ":" or "$" before sel_start
-	int i = sel_start - 1;
-	for (; i > sel_start - 6; i--)
-		if ((i >= 0 && debug_wstr[i] == L':' || debug_wstr[i] == L'$') && debug_wstr[i+3] != L'\n')
-			break;
-	if (i > sel_start - 6)
+	// check for the hex address value
+	for (int i = sel_start - 1; (i > sel_start - 6) && (i >= 0); i--)
 	{
-		wchar_t offsetBuffer[5];
-		wcsncpy(offsetBuffer, debug_wstr + i + 1, 4);
-		offsetBuffer[4] = 0;
-		// invalidate the string if a space or \r is found in it
-		wchar_t* firstspace = wcsstr(offsetBuffer, L" ");
-		if (!firstspace)
-			firstspace = wcsstr(offsetBuffer, L"\r");
-		if (!firstspace)
+		// find the first character before hex value
+		if (!((debug_wstr[i] >= '0' && debug_wstr[i] <= '9') || (debug_wstr[i] >= 'A' && debug_wstr[i] <= 'F')))
 		{
+			int hex_pos = i + 1;
+			int hex_len = 0;
 			unsigned int offset;
-			int numend;
-			if (swscanf(offsetBuffer, L"%4X", &offset) != EOF)
-			{
-				if (debug_wstr[i + 3] == L',' || debug_wstr[i+3] == L')')
-					numend = 3;
-				else
-					numend = 5;
-				// select the text
-				SendDlgItemMessage(hDebug, IDC_DEBUGGER_DISASSEMBLY, EM_SETSEL, (WPARAM)(i + 1), (LPARAM)(i + numend));
-				PrintOffsetToSeekAndBookmarkFields(offset);
-				return (int)offset;
-			}
+
+			// find length of the hex string
+			while (
+					(debug_wstr[hex_pos + hex_len] >= '0' && debug_wstr[hex_pos + hex_len] <= '9') ||
+					(debug_wstr[hex_pos + hex_len] >= 'A' && debug_wstr[hex_pos + hex_len] <= 'F')
+				) hex_len++;
+			// validate length of the value
+			if ((hex_len != 2) && (hex_len != 4)) break;
+			// validate symbol before the hex value
+			if (
+					(debug_wstr[i] != L':') && // ":XX" or ":XXXX"
+					(debug_wstr[i] != L'$') // "$XX" or "$XXXX"
+				) break;
+			// block "#$XX" pattern
+			if (
+					(i > 0) &&
+					(debug_wstr[i] == L'$') &&
+					(debug_wstr[i - 1] == L'#')
+				) break;
+			// validate symbol after the hex value
+			if (
+					((debug_wstr[hex_pos + hex_len] != L':') || (hex_len != 4)) && // opcode address
+					(debug_wstr[hex_pos + hex_len] != L',') &&
+					(debug_wstr[hex_pos + hex_len] != L')') &&
+					(debug_wstr[hex_pos + hex_len] != L' ') &&
+				    (debug_wstr[hex_pos + hex_len] != L'\n')
+				) break;			
+			if (swscanf(&debug_wstr[hex_pos], (hex_len == 2) ? L"%2X" : L"%4X", &offset) == EOF) break;
+			// select the text
+			SendDlgItemMessage(hDebug, IDC_DEBUGGER_DISASSEMBLY, EM_SETSEL, (WPARAM)(hex_pos), (LPARAM)(hex_pos + hex_len));
+			PrintOffsetToSeekAndBookmarkFields(offset);
+			return (int)offset;
 		}
 	}
 	
@@ -1762,7 +1774,7 @@ int Debugger_CheckClickingOnAnAddressOrSymbolicName(unsigned int lineNumber, boo
 		}
 
 		// then, try finding the name of disassembly_operands
-		for (i = disassembly_operands[lineNumber].size() - 1; i >= 0; i--)
+		for (int i = disassembly_operands[lineNumber].size() - 1; i >= 0; i--)
 		{
 			addr = disassembly_operands[lineNumber][i];
 			node = findNode(getNamesPointerForAddress(addr), addr);
@@ -1781,7 +1793,7 @@ int Debugger_CheckClickingOnAnAddressOrSymbolicName(unsigned int lineNumber, boo
 				{
 					// clicked on the operand name
 					// select the text
-					SendDlgItemMessage(hDebug, IDC_DEBUGGER_DISASSEMBLY, EM_SETSEL, (WPARAM)(int)(pos - debug_wstr), (LPARAM)((int)(pos - debug_wstr) + nameLen));
+					SendDlgItemMessage(hDebug, IDC_DEBUGGER_DISASSEMBLY, EM_SETSEL, (WPARAM)(int)(pos - debug_wstr), (LPARAM)((int)(pos - debug_wstr + nameLen - 1)));
 					PrintOffsetToSeekAndBookmarkFields(addr);
 					return (int)addr;
 				}
@@ -2410,7 +2422,7 @@ INT_PTR CALLBACK DebuggerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lP
 				int mouse_x = GET_X_LPARAM(lParam);
 				int mouse_y = GET_Y_LPARAM(lParam);
 				//mbg merge 7/18/06 changed pausing check
-				if (FCEUI_EmulationPaused() && (mouse_x > 6) && (mouse_x < 30) && (mouse_y > 10))
+				if ((mouse_x > 6) && (mouse_x < 30) && (mouse_y > 10))
 				{
 					int tmp = (mouse_y - 10) / debugSystem->disasmFontHeight;
 					if (tmp < (int)disassembly_addresses.size())
@@ -2740,7 +2752,7 @@ void UpdatePatcher(HWND hwndDlg){
 		EnableWindow(GetDlgItem(hwndDlg,IDC_ROMPATCHER_PATCH_DATA),FALSE);
 		EnableWindow(GetDlgItem(hwndDlg,IDC_ROMPATCHER_BTN_APPLY),FALSE);
 	}
-	if(GameInfo->type != GIT_CART)EnableWindow(GetDlgItem(hwndDlg,IDC_ROMPATCHER_BTN_SAVE),FALSE);
+	if((GameInfo->type != GIT_CART) && (GameInfo->type != GIT_VSUNI))EnableWindow(GetDlgItem(hwndDlg,IDC_ROMPATCHER_BTN_SAVE),FALSE);
 	else EnableWindow(GetDlgItem(hwndDlg,IDC_ROMPATCHER_BTN_SAVE),TRUE);
 }
 
