@@ -46,11 +46,11 @@ HWND hNTView;
 
 int NTViewPosX,NTViewPosY;
 
-static uint8 palcache[32]; //palette cache //mbg merge 7/19/06 needed to be static
+static uint8 palcache[36]; //palette cache
 int NTViewScanline=0,NTViewer=0;
 int NTViewSkip;
-int NTViewRefresh = 15;
-static int mouse_x,mouse_y; //todo: is static needed here? --mbg 7/19/06 - i think so
+int NTViewRefresh = 0;
+static int mouse_x,mouse_y;
 bool redrawtables = false;
 int chrchanged = 0;
 
@@ -83,23 +83,18 @@ NT_MirrorType ntmirroring, oldntmirroring = NT_NONE;
 
 int horzscroll, vertscroll;
 
-#define NTWIDTH			256
-#define NTHEIGHT		240
-//#define PATTERNBITWIDTH	PATTERNWIDTH*3
-#define NTDESTX		10
-#define NTDESTY		15
-#define ZOOM		1
+#define NTWIDTH             256
+#define NTHEIGHT            240
+#define NTDESTX_BASE        6
+#define NTDESTY_BASE        18
 
-//#define PALETTEWIDTH	32*4*4
-//#define PALETTEHEIGHT	32*2
-//#define PALETTEBITWIDTH	PALETTEWIDTH*3
-//#define PALETTEDESTX	10
-//#define PALETTEDESTY	327
+int ntDestX = NTDESTX_BASE;
+int ntDestY = NTDESTY_BASE;
 
 void UpdateMirroringButtons();
 void ChangeMirroring();
 
-static BITMAPINFO bmInfo; //todo is static needed here so it won't interefere with the pattern table viewer?
+static BITMAPINFO bmInfo;
 static HDC pDC;
 
 extern uint32 TempAddr, RefreshAddr;
@@ -107,6 +102,11 @@ extern uint8 XOffset;
 
 int xpos, ypos;
 int scrolllines = 1;
+int attview = 0;
+int hidepal = 0;
+
+// checkerboard tile for attribute view
+const uint8 ATTRIBUTE_VIEW_TILE[16] = {	0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF };
 
 //if you pass this 1 then it will draw no matter what. If you pass it 0
 //then it will draw if redrawtables is true
@@ -120,10 +120,10 @@ void NTViewDoBlit(int autorefresh) {
 
 	if((redrawtables && !autorefresh) || (autorefresh) || (scrolllines)){
 
-	BitBlt(pDC,NTDESTX,NTDESTY,NTWIDTH,NTHEIGHT,cache[0].hdc,0,0,SRCCOPY);
-	BitBlt(pDC,NTDESTX+NTWIDTH,NTDESTY,NTWIDTH,NTHEIGHT,cache[1].hdc,0,0,SRCCOPY);
-	BitBlt(pDC,NTDESTX,NTDESTY+NTHEIGHT,NTWIDTH,NTHEIGHT,cache[2].hdc,0,0,SRCCOPY);
-	BitBlt(pDC,NTDESTX+NTWIDTH,NTDESTY+NTHEIGHT,NTWIDTH,NTHEIGHT,cache[3].hdc,0,0,SRCCOPY);
+	BitBlt(pDC,ntDestX,ntDestY,NTWIDTH,NTHEIGHT,cache[0].hdc,0,0,SRCCOPY);
+	BitBlt(pDC,ntDestX+NTWIDTH,ntDestY,NTWIDTH,NTHEIGHT,cache[1].hdc,0,0,SRCCOPY);
+	BitBlt(pDC,ntDestX,ntDestY+NTHEIGHT,NTWIDTH,NTHEIGHT,cache[2].hdc,0,0,SRCCOPY);
+	BitBlt(pDC,ntDestX+NTWIDTH,ntDestY+NTHEIGHT,NTWIDTH,NTHEIGHT,cache[3].hdc,0,0,SRCCOPY);
 
 	redrawtables = false;
 	}
@@ -132,19 +132,15 @@ void NTViewDoBlit(int autorefresh) {
 		SetROP2(pDC,R2_NOT);
 
 		//draw vertical line
-		MoveToEx(pDC,NTDESTX+xpos,NTDESTY,NULL);
-		LineTo(pDC,NTDESTX+xpos,NTDESTY+(NTHEIGHT*2));
+		MoveToEx(pDC,ntDestX+xpos,ntDestY,NULL);
+		LineTo(pDC,ntDestX+xpos,ntDestY+(NTHEIGHT*2)-1);
 	
 		//draw horizontal line
-		MoveToEx(pDC,NTDESTX,NTDESTY+ypos,NULL);
-		LineTo(pDC,NTDESTX+(NTWIDTH*2),NTDESTY+ypos);
+		MoveToEx(pDC,ntDestX,ntDestY+ypos,NULL);
+		LineTo(pDC,ntDestX+(NTWIDTH*2)-1,ntDestY+ypos);
 
 		SetROP2(pDC,R2_COPYPEN);
 	}
-	//StretchBlt(pDC,NTDESTX,NTDESTY,NTWIDTH*ZOOM,NTHEIGHT*ZOOM,TmpDC0,0,NTHEIGHT-1,NTWIDTH,-NTHEIGHT,SRCCOPY);
-	//StretchBlt(pDC,NTDESTX+(NTWIDTH*ZOOM),NTDESTY,NTWIDTH*ZOOM,NTHEIGHT*ZOOM,TmpDC1,0,NTHEIGHT-1,NTWIDTH,-NTHEIGHT,SRCCOPY);
-	//StretchBlt(pDC,NTDESTX,NTDESTY+(NTHEIGHT*ZOOM),NTWIDTH*ZOOM,NTHEIGHT*ZOOM,TmpDC2,0,NTHEIGHT-1,NTWIDTH,-NTHEIGHT,SRCCOPY);
-	//StretchBlt(pDC,NTDESTX+(NTWIDTH*ZOOM),NTDESTY+(NTHEIGHT*ZOOM),NTWIDTH*ZOOM,NTHEIGHT*ZOOM,TmpDC3,0,NTHEIGHT-1,NTWIDTH,-NTHEIGHT,SRCCOPY);
 }
 
 void UpdateMirroringButtons(){
@@ -193,7 +189,7 @@ void ChangeMirroring(){
 	return;
 }
 
-INLINE void DrawChr(uint8 *pbitmap,uint8 *chr,int pal){
+INLINE void DrawChr(uint8 *pbitmap,const uint8 *chr,int pal){
 	int y, x, tmp, index=0, p=0;
 	uint8 chr0, chr1;
 	//uint8 *table = &VPage[0][0]; //use the background table
@@ -279,9 +275,13 @@ void DrawNameTable(int scanline, int ntnum, bool invalidateCache) {
 				int refreshaddr = (x)+(y)*32;
 
 				a = FCEUPPU_GetAttr(ntnum,x,y);
+				if (hidepal) a = 8;
+
+				const uint8* chrp = FCEUPPU_GetCHR(ptable+chr,refreshaddr);
+				if (attview) chrp = ATTRIBUTE_VIEW_TILE;
 
 				//a good way to do it:
-				DrawChr(pbitmap,FCEUPPU_GetCHR(ptable+chr,refreshaddr),a);
+				DrawChr(pbitmap,chrp,a);
 
 				tablecache[ntaddr] = table[ntaddr];
 				tablecache[attraddr] = table[attraddr];
@@ -301,6 +301,77 @@ void DrawNameTable(int scanline, int ntnum, bool invalidateCache) {
 	//}
 }
 
+static void CalculateBitmapPositions(HWND hwndDlg)
+{
+	RECT rect;
+	POINT pt;
+	GetWindowRect(GetDlgItem(hwndDlg, IDC_NTVIEW_TABLE_BOX), &rect);
+	pt.x = rect.left;
+	pt.y = rect.top;
+	ScreenToClient(hwndDlg, &pt);
+	ntDestX = pt.x + NTDESTX_BASE;
+	ntDestY = pt.y + NTDESTY_BASE;
+}
+
+static BOOL CALLBACK EnsurePixelSizeEnumWindowsProc(HWND hwnd, LPARAM lParam)
+{
+	const int shift = lParam;
+	HWND ntbox = GetDlgItem(hNTView, IDC_NTVIEW_TABLE_BOX);
+
+	if (hwnd != ntbox)
+	{
+		RECT rect;
+		GetWindowRect(hwnd, &rect);
+		ScreenToClient(hNTView,(LPPOINT)&rect);
+		ScreenToClient(hNTView,((LPPOINT)&rect)+1);
+		SetWindowPos(hwnd,0,rect.left,rect.top+shift,0,0,SWP_NOZORDER | SWP_NOSIZE);
+	}
+	return TRUE;
+}
+
+static void EnsurePixelSize()
+{
+	// DPI varies, so the pixel size of the window may be too small to fit the viewer.
+	// This expands the window and moves its controls around if necessary.
+
+	if (!hNTView) return;
+	HWND ntbox = GetDlgItem(hNTView, IDC_NTVIEW_TABLE_BOX);
+
+	const int MIN_W = (NTWIDTH  * 2) + (NTDESTX_BASE * 2);
+	const int MIN_H = (NTHEIGHT * 2) + (NTDESTY_BASE + 8);
+
+	RECT rect, client;
+	GetWindowRect(ntbox,&rect);
+	GetClientRect(ntbox,&client);
+
+	int nt_w = rect.right - rect.left;
+	int nt_h = rect.bottom - rect.top;
+
+	int nt_w_add = (nt_w < MIN_W) ? (MIN_W - nt_w) : 0;
+	int nt_h_add = (nt_h < MIN_H) ? (MIN_H - nt_h) : 0;
+
+	if (nt_w_add <= 0 && nt_h_add <= 0) return;
+
+	// expand parent window
+	RECT wrect;
+	GetWindowRect(hNTView,&wrect);
+	int ww = (wrect.right - wrect.left) + nt_w_add;
+	int wh = (wrect.bottom - wrect.top) + nt_h_add;
+	SetWindowPos(hNTView,0,0,0,ww,wh,SWP_NOZORDER | SWP_NOMOVE);
+
+	// expand NT box
+	SetWindowPos(ntbox,0,0,0,nt_w + nt_w_add,nt_h + nt_h_add, SWP_NOZORDER | SWP_NOMOVE);
+
+	// move children
+	if (nt_h_add > 0)
+	{
+		EnumChildWindows(hNTView, EnsurePixelSizeEnumWindowsProc, nt_h_add);
+	}
+
+	CalculateBitmapPositions(hNTView);
+	RedrawWindow(hNTView,0,0,RDW_ERASE | RDW_INVALIDATE);
+}
+
 void FCEUD_UpdateNTView(int scanline, bool drawall) {
 	if(!NTViewer) return;
 	if(scanline != -1 && scanline != NTViewScanline) return;
@@ -311,11 +382,6 @@ void FCEUD_UpdateNTView(int scanline, bool drawall) {
 	ppu_getScroll(xpos,ypos);
 
 
-	//char str[50];
-	//sprintf(str,"%d,%d,%d",horzscroll,vertscroll,ntmirroring);
-	//sprintf(str,"%08X  %08X",TempAddr, RefreshAddr);
-	//SetDlgItemText(hNTView, IDC_NTVIEW_PROPERTIES_LINE_1, str);
-
 	if (NTViewSkip < NTViewRefresh) return;
 
 	if(chrchanged) drawall = 1;
@@ -369,10 +435,10 @@ void KillNTView() {
 	NTViewSkip=0;
 }
 
-BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
+INT_PTR CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
 	RECT wrect;
 	char str[50];
-	int TileID, TileX, TileY, NameTable, PPUAddress;
+	int TileID, TileX, TileY, NameTable, PPUAddress, AttAddress, Attrib;
 
 	switch(uMsg) {
 		case WM_INITDIALOG:
@@ -380,6 +446,8 @@ BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 			if (NTViewPosY==-32000) NTViewPosY=0;
 			SetWindowPos(hwndDlg,0,NTViewPosX,NTViewPosY,0,0,SWP_NOSIZE|SWP_NOZORDER|SWP_NOOWNERZORDER);
 
+			CalculateBitmapPositions(hwndDlg);
+
 			//prepare the bitmap attributes
 			//pattern tables
 			memset(&bmInfo.bmiHeader,0,sizeof(BITMAPINFOHEADER));
@@ -390,7 +458,7 @@ BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 			bmInfo.bmiHeader.biBitCount = 24;
 
 			//create memory dcs
-			pDC = GetDC(hwndDlg); // GetDC(GetDlgItem(hwndDlg,IDC_NTVIEW_TABLE_BOX));
+			pDC = GetDC(hwndDlg);
 			for(int i=0;i<4;i++) {
 				NTCache &c = cache[i];
 				c.hdc = CreateCompatibleDC(pDC);
@@ -410,13 +478,20 @@ BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 
 			SelectObject (pDC, CreatePen (PS_SOLID, 2, RGB (255, 255, 255))) ;
 
-			CheckDlgButton(hwndDlg, IDC_NTVIEW_SHOW_SCROLL_LINES, BST_CHECKED);
+			CheckDlgButton(hwndDlg, IDC_NTVIEW_SHOW_SCROLL_LINES, scrolllines ? BST_CHECKED : BST_UNCHECKED);
+			EnableWindow(GetDlgItem(hwndDlg, IDC_NTVIEW_SCANLINE_TEXT), scrolllines);
+			EnableWindow(GetDlgItem(hwndDlg, IDC_NTVIEW_SCANLINE), scrolllines);
+			CheckDlgButton(hwndDlg, IDC_NTVIEW_SHOW_ATTRIBUTES,   attview ? BST_CHECKED : BST_UNCHECKED);
+			CheckDlgButton(hwndDlg, IDC_NTVIEW_HIDE_PALETTES,     hidepal ? BST_CHECKED : BST_UNCHECKED);
+
 			//clear cache
-			//memset(palcache,0,32);
-			//memset(ntcache0,0,0x400);
-			//memset(ntcache1,0,0x400);
-			//memset(ntcache2,0,0x400);
-			//memset(ntcache3,0,0x400);
+			memset(palcache,0,32);
+
+			// forced palette (e.g. for debugging nametables when palettes are all-black)
+			palcache[(8*4)+0] = 0x0F;
+			palcache[(8*4)+1] = 0x00;
+			palcache[(8*4)+2] = 0x10;
+			palcache[(8*4)+3] = 0x20;
 
 			NTViewer=1;
 			break;
@@ -444,24 +519,14 @@ BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 		case WM_RBUTTONDOWN:
 			mouse_x = GET_X_LPARAM(lParam);
 			mouse_y = GET_Y_LPARAM(lParam);
-			/*if (((mouse_x >= PATTERNDESTX) && (mouse_x < (PATTERNDESTX+(PATTERNWIDTH*ZOOM)))) && (mouse_y >= PATTERNDESTY) && (mouse_y < (PATTERNDESTY+(PATTERNHEIGHT*ZOOM)))) {
-				if (pindex0 == 7) pindex0 = 0;
-				else pindex0++;
-			}
-			else if (((mouse_x >= PATTERNDESTX+(PATTERNWIDTH*ZOOM)+1) && (mouse_x < (PATTERNDESTX+(PATTERNWIDTH*ZOOM)*2+1))) && (mouse_y >= PATTERNDESTY) && (mouse_y < (PATTERNDESTY+(PATTERNHEIGHT*ZOOM)))) {
-				if (pindex1 == 7) pindex1 = 0;
-				else pindex1++;
-			}
-			UpdatePPUView(0);
-			PPUViewDoBlit();*/
 			break;
 		case WM_MOUSEMOVE:
 			mouse_x = GET_X_LPARAM(lParam);
 			mouse_y = GET_Y_LPARAM(lParam);
-			if((mouse_x > NTDESTX) && (mouse_x < NTDESTX+(NTWIDTH*2))
-				&& (mouse_y > NTDESTY) && (mouse_y < NTDESTY+(NTHEIGHT*2))){
-				TileX = (mouse_x-NTDESTX)/8;
-				TileY = (mouse_y-NTDESTY)/8;
+			if((mouse_x > ntDestX) && (mouse_x < ntDestX+(NTWIDTH*2))
+				&& (mouse_y > ntDestY) && (mouse_y < ntDestY+(NTHEIGHT*2))){
+				TileX = (mouse_x-ntDestX)/8;
+				TileY = (mouse_y-ntDestY)/8;
 				sprintf(str,"X / Y: %0d / %0d",TileX,TileY);
 				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_2,str);
 				NameTable = (TileX/32)+((TileY/30)*2);
@@ -471,38 +536,12 @@ BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 				TileID = vnapage[(PPUAddress>>10)&0x3][PPUAddress&0x3FF];
 				sprintf(str,"Tile ID: %02X",TileID);
 				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_1,str);
+				AttAddress = 0x23C0 | (PPUAddress & 0x0C00) | ((PPUAddress >> 4) & 0x38) | ((PPUAddress >> 2) & 0x07);
+				Attrib = vnapage[(AttAddress>>10)&0x3][AttAddress&0x3FF];
+				Attrib = (Attrib >> ((PPUAddress&2) | ((PPUAddress&64)>>4))) & 0x3;
+				sprintf(str,"Attribute: %1X (%04X)",Attrib,AttAddress);
+				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_4,str);
 			}
-
-/*			if (((mouse_x >= PATTERNDESTX) && (mouse_x < (PATTERNDESTX+(PATTERNWIDTH*ZOOM)))) && (mouse_y >= PATTERNDESTY) && (mouse_y < (PATTERNDESTY+(PATTERNHEIGHT*ZOOM)))) {
-				mouse_x = (mouse_x-PATTERNDESTX)/(8*ZOOM);
-				mouse_y = (mouse_y-PATTERNDESTY)/(8*ZOOM);
-				sprintf(str,"Tile: $%X%X",mouse_y,mouse_x);
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_1,str);
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_2,"Tile:");
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_3,"Palettes");
-			}
-			else if (((mouse_x >= PATTERNDESTX+(PATTERNWIDTH*ZOOM)+1) && (mouse_x < (PATTERNDESTX+(PATTERNWIDTH*ZOOM)*2+1))) && (mouse_y >= PATTERNDESTY) && (mouse_y < (PATTERNDESTY+(PATTERNHEIGHT*ZOOM)))) {
-				mouse_x = (mouse_x-(PATTERNDESTX+(PATTERNWIDTH*ZOOM)+1))/(8*ZOOM);
-				mouse_y = (mouse_y-PATTERNDESTY)/(8*ZOOM);
-				sprintf(str,"Tile: $%X%X",mouse_y,mouse_x);
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_2,str);
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_1,"Tile:");
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_3,"Palettes");
-			}
-			else if (((mouse_x >= PALETTEDESTX) && (mouse_x < (PALETTEDESTX+PALETTEWIDTH))) && (mouse_y >= PALETTEDESTY) && (mouse_y < (PALETTEDESTY+PALETTEHEIGHT))) {
-				mouse_x = (mouse_x-PALETTEDESTX)/32;
-				mouse_y = (mouse_y-PALETTEDESTY)/32;
-				sprintf(str,"Palette: $%02X",palcache[(mouse_y<<4)|mouse_x]);
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_1,"Tile:");
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_2,"Tile:");
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_3,str);
-			}
-			else {
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_1,"Tile:");
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_2,"Tile:");
-				SetDlgItemText(hwndDlg,IDC_NTVIEW_PROPERTIES_LINE_3,"Palettes");
-			}
-*/
 			break;
 		case WM_NCACTIVATE:
 			sprintf(str,"%d",NTViewScanline);
@@ -530,6 +569,16 @@ BOOL CALLBACK NTViewCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 							break;
 						case IDC_NTVIEW_SHOW_SCROLL_LINES : 
 							scrolllines ^= 1;
+							EnableWindow(GetDlgItem(hwndDlg, IDC_NTVIEW_SCANLINE_TEXT), scrolllines);
+							EnableWindow(GetDlgItem(hwndDlg, IDC_NTVIEW_SCANLINE), scrolllines);
+							chrchanged = 1;
+							break;
+						case IDC_NTVIEW_SHOW_ATTRIBUTES :
+							attview ^= 1;
+							chrchanged = 1;
+							break;
+						case IDC_NTVIEW_HIDE_PALETTES :
+							hidepal ^= 1;
 							chrchanged = 1;
 							break;
 					}
@@ -559,6 +608,7 @@ void DoNTView()
 	if (!hNTView)
 	{
 		hNTView = CreateDialog(fceu_hInstance,"NTVIEW",NULL,NTViewCallB);
+		EnsurePixelSize();
 		new(cache) NTCache[4]; //reinitialize NTCache
 	}
 	if (hNTView)
