@@ -126,7 +126,10 @@ void UpdateReplayCommentsSubs(const char * fname) {
 	MOVIE_INFO info;
 	
 	FCEUFILE *fp = FCEU_fopen(fname,0,"rb",0);
-	fp->stream = fp->stream->memwrap();
+	EMUFILE *tmp = fp->stream->memwrap();
+	if (tmp != fp->stream)
+		delete fp->stream;
+	fp->stream = tmp;
 	bool scanok = FCEUI_MovieGetInfo(fp, info, true);
 	delete fp;
 
@@ -151,8 +154,11 @@ void UpdateReplayDialog(HWND hwndDlg)
 	{
 		MOVIE_INFO info;
 
-		FCEUFILE* fp = FCEU_fopen(fn,0,"rb",0);
-		fp->stream = fp->stream->memwrap();
+		FCEUFILE *fp = FCEU_fopen(fn,0,"rb",0);
+		EMUFILE *tmp = fp->stream->memwrap();
+		if (tmp != fp->stream)
+			delete fp->stream;
+		fp->stream = tmp;
 		bool isarchive = FCEU_isFileInArchive(fn);
 		bool ismovie = FCEUI_MovieGetInfo(fp, info, false);
 		delete fp;
@@ -263,11 +269,6 @@ void UpdateReplayDialog(HWND hwndDlg)
 
 		free(fn);
 	}
-	else
-	{
-		EnableWindow(GetDlgItem(hwndDlg,IDC_EDIT_OFFSET),FALSE);
-		EnableWindow(GetDlgItem(hwndDlg,IDC_EDIT_FROM),FALSE);
-	}
 
 	if(doClear)
 	{
@@ -283,6 +284,8 @@ void UpdateReplayDialog(HWND hwndDlg)
 		EnableWindow(GetDlgItem(hwndDlg,IDC_CHECK_READONLY),FALSE);
 		EnableWindow(GetDlgItem(hwndDlg,IDOK),FALSE);
 	}
+	EnableWindow(GetDlgItem(hwndDlg,IDC_CHECK_STOPMOVIE), !doClear);
+	EnableWindow(GetDlgItem(hwndDlg,IDC_EDIT_STOPFRAME), !doClear && SendDlgItemMessage(hwndDlg, IDC_CHECK_STOPMOVIE, BM_GETCHECK, 0, 0) == BST_CHECKED);
 }
 
 // C:\fceu\movies\bla.fcm  +  C:\fceu\fceu\  ->  C:\fceu\movies\bla.fcm
@@ -320,7 +323,7 @@ void AbsoluteToRelative(char *const dst, const char *const dir, const char *cons
 }
 
 
-BOOL CALLBACK ReplayMetadataDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK ReplayMetadataDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	RECT wrect;
 	switch(uMsg)
@@ -499,7 +502,7 @@ void HandleScan(HWND hwndDlg, FCEUFILE* file, int& i)
 	SendDlgItemMessage(hwndDlg, IDC_COMBO_FILENAME, CB_INSERTSTRING, i++, (LPARAM)relative);
 }
 
-BOOL CALLBACK ReplayDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK ReplayDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	switch(uMsg)
 	{
@@ -643,6 +646,9 @@ BOOL CALLBACK ReplayDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lP
 			int wID = LOWORD(wParam);
 			switch(wID)
 			{
+			case IDC_CHECK_STOPMOVIE:
+				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_STOPFRAME), SendDlgItemMessage(hwndDlg, IDC_CHECK_STOPMOVIE, BM_GETCHECK, 0, 0) == BST_CHECKED);
+				break;
 			case IDC_BUTTON_METADATA:
 				DialogBoxParam(fceu_hInstance, "IDD_REPLAY_METADATA", hwndDlg, ReplayMetadataDialogProc, (LPARAM)0);
 				break;
@@ -708,6 +714,23 @@ BOOL CALLBACK ReplayDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lP
 						}
 						else
 						{
+							extern unsigned int FrozenAddressCount;
+							if(FrozenAddressCount)
+							{
+								char ch[512];
+								sprintf(ch, "You have %d activated cheats. If this is not your intentional, it can cause playback prblems! Do you want to disable all of them and continue?", FrozenAddressCount);
+								switch(MessageBox(hwndDlg, ch, "Movie playing problem", MB_YESNOCANCEL|MB_ICONEXCLAMATION))
+								{
+									case IDCANCEL:
+										return TRUE;
+									case IDYES:
+										extern void DisableAllCheats();
+										DisableAllCheats();
+										extern void UpdateCheatRelatedWindow();
+										UpdateCheatRelatedWindow();
+								}
+
+							}
 							// user had made their choice
 							// TODO: warn the user when they open a movie made with a different ROM
 							char* fn=GetReplayPath(hwndDlg);
@@ -744,7 +767,7 @@ BOOL CALLBACK ReplayDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lP
 			else
 				SetTextColor(hdcStatic, RGB(255,0,0));		// use red for a mismatch
 			SetBkMode((HDC)wParam,TRANSPARENT);
-			return (BOOL)GetSysColorBrush(COLOR_BTNFACE);
+			return (INT_PTR)GetSysColorBrush(COLOR_BTNFACE);
 		} else if ((HWND)lParam == GetDlgItem(hwndDlg, IDC_LABEL_NEWPPUUSED))
 		{
 			HDC hdcStatic = (HDC)wParam;
@@ -757,7 +780,7 @@ BOOL CALLBACK ReplayDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lP
 			else
 				SetTextColor(hdcStatic, RGB(255,0,0));		// use red for a mismatch
 			SetBkMode((HDC)wParam,TRANSPARENT);
-			return (BOOL)GetSysColorBrush(COLOR_BTNFACE);
+			return (INT_PTR)GetSysColorBrush(COLOR_BTNFACE);
 		} else
 		{
 			return FALSE;
@@ -818,7 +841,7 @@ static void UpdateRecordDialogPath(HWND hwndDlg, const std::string &fname)
 	}
 }
 
-static BOOL CALLBACK RecordDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+static INT_PTR CALLBACK RecordDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	static struct CreateMovieParameters* p = NULL;
 
@@ -851,6 +874,7 @@ static BOOL CALLBACK RecordDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LP
 
 			SendDlgItemMessage(hwndDlg, IDC_COMBO_RECORDFROM, CB_INSERTSTRING, i++, (LPARAM)"Start");
 			SendDlgItemMessage(hwndDlg, IDC_COMBO_RECORDFROM, CB_INSERTSTRING, i++, (LPARAM)"Now");
+			SendDlgItemMessage(hwndDlg, IDC_COMBO_RECORDFROM, CB_INSERTSTRING, i++, (LPARAM)"SaveRam");
 
 			memset(&wfd, 0, sizeof(wfd));
 			hFind = FindFirstFile(findGlob, &wfd);
@@ -928,6 +952,23 @@ static BOOL CALLBACK RecordDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LP
 			{
 			case IDOK:
 				{
+					extern unsigned int FrozenAddressCount;
+					if (FrozenAddressCount)
+					{
+						char ch[512];
+						sprintf(ch, "You have %d activated cheats. If this is not your intentional, it can cause playback prblems! Do you want to disable all of them and continue?", FrozenAddressCount);
+						switch (MessageBox(hwndDlg, ch, "Movie recording problem", MB_YESNOCANCEL | MB_ICONEXCLAMATION))
+						{
+							case IDCANCEL:
+								return TRUE;
+							case IDYES:
+								extern void DisableAllCheats();
+								DisableAllCheats();
+								extern void UpdateCheatRelatedWindow();
+								UpdateCheatRelatedWindow();
+						}
+					}
+
 					LONG lIndex = SendDlgItemMessage(hwndDlg, IDC_COMBO_RECORDFROM, CB_GETCURSEL, 0, 0);
 					p->szFilename = GetRecordPath(hwndDlg);
 					p->recordFrom = (int)lIndex;
@@ -941,7 +982,7 @@ static BOOL CALLBACK RecordDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LP
 						// convert Unicode wstring to UTF8 char* string
 						WideCharToMultiByte(CP_UTF8, 0, (p->authorName).c_str(), -1, taseditorConfig.lastAuthorName, AUTHOR_NAME_MAX_LEN, 0, 0);
 
-					if (lIndex >= 2)
+					if (lIndex >= 3)
 						p->szSavestateFilename = GetSavePath(hwndDlg);
 					EndDialog(hwndDlg, 1);
 				}
@@ -983,11 +1024,11 @@ void FCEUD_MovieRecordTo()
 	if (!GameInfo) return;
 	static struct CreateMovieParameters p;
 	p.szFilename = strdup(FCEU_MakeFName(FCEUMKF_MOVIE,0,0).c_str());
-	if(p.recordFrom >= 2) p.recordFrom=1;
+	if(p.recordFrom >= 3) p.recordFrom=1;
 
 	if(DialogBoxParam(fceu_hInstance, "IDD_RECORDINP", hAppWnd, RecordDialogProc, (LPARAM)&p))
 	{
-		if(p.recordFrom >= 2)
+		if(p.recordFrom >= 3)
 		{
 			// attempt to load the savestate
 			// FIXME:  pop open a messagebox if this fails
@@ -1006,6 +1047,7 @@ void FCEUD_MovieRecordTo()
 
 		EMOVIE_FLAG flags = MOVIE_FLAG_NONE;
 		if(p.recordFrom == 0) flags = MOVIE_FLAG_FROM_POWERON;
+		if(p.recordFrom == 2) flags = MOVIE_FLAG_FROM_SAVERAM;
 		FCEUI_SaveMovie(p.szFilename.c_str(), flags, p.authorName);
 	}
 }
