@@ -53,12 +53,12 @@ extern bool mustCallManualLuaFunction;
 
 extern char* GetKeyComboName(int c);
 
-extern BOOL CALLBACK findNoteWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
-extern BOOL CALLBACK aboutWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
-extern BOOL CALLBACK savingOptionsWndProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
+extern INT_PTR CALLBACK findNoteWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
+extern INT_PTR CALLBACK aboutWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
+extern INT_PTR CALLBACK savingOptionsWndProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
 
 // main window wndproc
-BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
+INT_PTR CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
 // wndprocs for "Marker X" text fields
 LRESULT APIENTRY IDC_PLAYBACK_MARKER_WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
 LRESULT APIENTRY IDC_SELECTION_MARKER_WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
@@ -132,47 +132,47 @@ char taseditorHelpFilename[] = "\\taseditor.chm";
 // "y < 0" means that the coordinate is counted from the lower border of the window (bottom-aligned)
 // The items in this array MUST be sorted by the same order as the Window_items_enum!
 WindowItemData windowItems[TASEDITOR_WINDOW_TOTAL_ITEMS] = {
-	WINDOWITEMS_PIANO_ROLL, IDC_LIST1, 0, 0, -1, -1, "", "", false, 0, 0,
-	WINDOWITEMS_PLAYBACK_MARKER, IDC_PLAYBACK_MARKER, 0, 0, 0, 0, "Click here to scroll Piano Roll to Playback cursor (hotkey: tap Shift twice)", "", false, 0, 0,
-	WINDOWITEMS_PLAYBACK_MARKER_EDIT, IDC_PLAYBACK_MARKER_EDIT, 0, 0, -1, 0, "Click to edit text", "", false, 0, 0,
-	WINDOWITEMS_SELECTION_MARKER, IDC_SELECTION_MARKER, 0, -1, 0, -1, "Click here to scroll Piano Roll to Selection (hotkey: tap Ctrl twice)", "", false, 0, 0,
-	WINDOWITEMS_SELECTION_MARKER_EDIT, IDC_SELECTION_MARKER_EDIT, 0, -1, -1, -1, "Click to edit text", "", false, 0, 0,
-	WINDOWITEMS_PLAYBACK_BOX, IDC_PLAYBACK_BOX, -1, 0, 0, 0, "", "", false, 0, 0,
-	WINDOWITEMS_PROGRESS_BUTTON, IDC_PROGRESS_BUTTON, -1, 0, 0, 0, "Click here when you want to abort seeking", "", false, EMUCMD_TASEDITOR_CANCEL_SEEKING, 0,
-	WINDOWITEMS_REWIND_FULL, TASEDITOR_REWIND_FULL, -1, 0, 0, 0, "Send Playback to previous Marker (mouse: Shift+Wheel up) (hotkey: Shift+PageUp)", "", false, 0, 0,
-	WINDOWITEMS_REWIND, TASEDITOR_REWIND, -1, 0, 0, 0, "Rewind 1 frame (mouse: Right button+Wheel up) (hotkey: Shift+Up)", "", false, EMUCMD_TASEDITOR_REWIND, 0,
-	WINDOWITEMS_PAUSE, TASEDITOR_PLAYSTOP, -1, 0, 0, 0, "Pause/Unpause Emulation (mouse: Middle button)", "", false, EMUCMD_PAUSE, 0,
-	WINDOWITEMS_FORWARD, TASEDITOR_FORWARD, -1, 0, 0, 0, "Advance 1 frame (mouse: Right button+Wheel down) (hotkey: Shift+Down)", "", false, EMUCMD_FRAME_ADVANCE, 0,
-	WINDOWITEMS_FORWARD_FULL, TASEDITOR_FORWARD_FULL, -1, 0, 0, 0, "Send Playback to next Marker (mouse: Shift+Wheel down) (hotkey: Shift+PageDown)", "", false, 0, 0,
-	WINDOWITEMS_PROGRESS_BAR, IDC_PROGRESS1, -1, 0, 0, 0, "", "", false, 0, 0,
-	WINDOWITEMS_FOLLOW_CURSOR, CHECK_FOLLOW_CURSOR, -1, 0, 0, 0, "The Piano Roll will follow Playback cursor movements", "", false, 0, 0,
-	WINDOWITEMS_TURBO_SEEK, CHECK_TURBO_SEEK, -1, 0, 0, 0, "Uncheck when you need to watch seeking in slow motion", "", false, 0, 0,
-	WINDOWITEMS_AUTORESTORE_PLAYBACK, CHECK_AUTORESTORE_PLAYBACK, -1, 0, 0, 0, "Whenever you change Input above Playback cursor, the cursor returns to where it was before the change", "", false, EMUCMD_TASEDITOR_SWITCH_AUTORESTORING, 0,
-	WINDOWITEMS_RECORDER_BOX, IDC_RECORDER_BOX, -1, 0, 0, 0, "", "", false, 0, 0,
-	WINDOWITEMS_RECORDING, IDC_RECORDING, -1, 0, 0, 0, "Switch Input Recording on/off", "", false, EMUCMD_MOVIE_READONLY_TOGGLE, 0,
-	WINDOWITEMS_RECORD_ALL, IDC_RADIO_ALL, -1, 0, 0, 0, "Switch off Multitracking", "", false, 0, 0,
-	WINDOWITEMS_RECORD_1P, IDC_RADIO_1P, -1, 0, 0, 0, "Select Joypad 1 as current", "", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
-	WINDOWITEMS_RECORD_2P, IDC_RADIO_2P, -1, 0, 0, 0, "Select Joypad 2 as current", "", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
-	WINDOWITEMS_RECORD_3P, IDC_RADIO_3P, -1, 0, 0, 0, "Select Joypad 3 as current", "", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
-	WINDOWITEMS_RECORD_4P, IDC_RADIO_4P, -1, 0, 0, 0, "Select Joypad 4 as current", "", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
-	WINDOWITEMS_SUPERIMPOSE, IDC_SUPERIMPOSE, -1, 0, 0, 0, "Allows to superimpose old Input with new buttons, instead of overwriting", "", false, 0, 0,
-	WINDOWITEMS_USE_PATTERN, IDC_USEPATTERN, -1, 0, 0, 0, "Applies current Autofire Pattern to Input recording", "", false, 0, 0,
-	WINDOWITEMS_SPLICER_BOX, IDC_SPLICER_BOX, -1, 0, 0, 0, "", "", false, 0, 0,
-	WINDOWITEMS_SELECTION_TEXT, IDC_TEXT_SELECTION, -1, 0, 0, 0, "Current size of Selection", "", false, 0, 0,
-	WINDOWITEMS_CLIPBOARD_TEXT, IDC_TEXT_CLIPBOARD, -1, 0, 0, 0, "Current size of Input in the Clipboard", "", false, 0, 0,
-	WINDOWITEMS_LUA_BOX, IDC_LUA_BOX, -1, 0, 0, 0, "", "", false, 0, 0,
-	WINDOWITEMS_RUN_MANUAL, TASEDITOR_RUN_MANUAL, -1, 0, 0, 0, "Press the button to execute Lua Manual Function", "", false, EMUCMD_TASEDITOR_RUN_MANUAL_LUA, 0,
-	WINDOWITEMS_RUN_AUTO, IDC_RUN_AUTO, -1, 0, 0, 0, "Enable Lua Auto Function (but first it must be registered by Lua script)", "", false, 0, 0,
-	WINDOWITEMS_BRANCHES_BUTTON, IDC_BRANCHES_BUTTON, -1, 0, 0, 0, "Click here to switch between Bookmarks List and Branches Tree", "", false, 0, 0,
-	WINDOWITEMS_BOOKMARKS_BOX, IDC_BOOKMARKS_BOX, -1, 0, 0, 0, "", "", false, 0, 0,
-	WINDOWITEMS_BOOKMARKS_LIST, IDC_BOOKMARKSLIST, -1, 0, 0, 0, "Right click = set Bookmark, Left click = jump to Bookmark or load Branch", "", false, 0, 0,
-	WINDOWITEMS_BRANCHES_BITMAP, IDC_BRANCHES_BITMAP, -1, 0, 0, 0, "Right click = set Bookmark, single Left click = jump to Bookmark, double Left click = load Branch", "", false, 0, 0,
-	WINDOWITEMS_HISTORY_BOX, IDC_HISTORY_BOX, -1, 0, 0, -1, "", "", false, 0, 0,
-	WINDOWITEMS_HISTORY_LIST, IDC_HISTORYLIST, -1, 0, 0, -1, "Click to revert the project back to that time", "", false, 0, 0,
-	WINDOWITEMS_PREVIOUS_MARKER, TASEDITOR_PREV_MARKER, -1, -1, 0, -1, "Send Selection to previous Marker (mouse: Ctrl+Wheel up) (hotkey: Ctrl+PageUp)", "", false, 0, 0,
-	WINDOWITEMS_SIMILAR, TASEDITOR_FIND_BEST_SIMILAR_MARKER, -1, -1, 0, -1, "Auto-search for Marker Note", "", false, 0, 0,
-	WINDOWITEMS_MORE, TASEDITOR_FIND_NEXT_SIMILAR_MARKER, -1, -1, 0, -1, "Continue Auto-search", "", false, 0, 0,
-	WINDOWITEMS_NEXT_MARKER, TASEDITOR_NEXT_MARKER, -1, -1, 0, -1, "Send Selection to next Marker (mouse: Ctrl+Wheel up) (hotkey: Ctrl+PageDown)", "", false, 0, 0,
+	WINDOWITEMS_PIANO_ROLL, IDC_LIST1, 0, 0, -1, -1, "", false, 0, 0,
+	WINDOWITEMS_PLAYBACK_MARKER, IDC_PLAYBACK_MARKER, 0, 0, 0, 0, "Click here to scroll Piano Roll to Playback cursor (hotkey: tap Shift twice)", false, 0, 0,
+	WINDOWITEMS_PLAYBACK_MARKER_EDIT, IDC_PLAYBACK_MARKER_EDIT, 0, 0, -1, 0, "Click to edit text", false, 0, 0,
+	WINDOWITEMS_SELECTION_MARKER, IDC_SELECTION_MARKER, 0, -1, 0, -1, "Click here to scroll Piano Roll to Selection (hotkey: tap Ctrl twice)", false, 0, 0,
+	WINDOWITEMS_SELECTION_MARKER_EDIT, IDC_SELECTION_MARKER_EDIT, 0, -1, -1, -1, "Click to edit text", false, 0, 0,
+	WINDOWITEMS_PLAYBACK_BOX, IDC_PLAYBACK_BOX, -1, 0, 0, 0, "", false, 0, 0,
+	WINDOWITEMS_PROGRESS_BUTTON, IDC_PROGRESS_BUTTON, -1, 0, 0, 0, "Click here when you want to abort seeking", false, EMUCMD_TASEDITOR_CANCEL_SEEKING, 0,
+	WINDOWITEMS_REWIND_FULL, TASEDITOR_REWIND_FULL, -1, 0, 0, 0, "Send Playback to previous Marker (mouse: Shift+Wheel up) (hotkey: Shift+PageUp)", false, 0, 0,
+	WINDOWITEMS_REWIND, TASEDITOR_REWIND, -1, 0, 0, 0, "Rewind 1 frame (mouse: Right button+Wheel up) (hotkey: Shift+Up)", false, EMUCMD_TASEDITOR_REWIND, 0,
+	WINDOWITEMS_PAUSE, TASEDITOR_PLAYSTOP, -1, 0, 0, 0, "Pause/Unpause Emulation (mouse: Middle button)", false, EMUCMD_PAUSE, 0,
+	WINDOWITEMS_FORWARD, TASEDITOR_FORWARD, -1, 0, 0, 0, "Advance 1 frame (mouse: Right button+Wheel down) (hotkey: Shift+Down)", false, EMUCMD_FRAME_ADVANCE, 0,
+	WINDOWITEMS_FORWARD_FULL, TASEDITOR_FORWARD_FULL, -1, 0, 0, 0, "Send Playback to next Marker (mouse: Shift+Wheel down) (hotkey: Shift+PageDown)", false, 0, 0,
+	WINDOWITEMS_PROGRESS_BAR, IDC_PROGRESS1, -1, 0, 0, 0, "", false, 0, 0,
+	WINDOWITEMS_FOLLOW_CURSOR, CHECK_FOLLOW_CURSOR, -1, 0, 0, 0, "The Piano Roll will follow Playback cursor movements", false, 0, 0,
+	WINDOWITEMS_TURBO_SEEK, CHECK_TURBO_SEEK, -1, 0, 0, 0, "Uncheck when you need to watch seeking in slow motion", false, 0, 0,
+	WINDOWITEMS_AUTORESTORE_PLAYBACK, CHECK_AUTORESTORE_PLAYBACK, -1, 0, 0, 0, "Whenever you change Input above Playback cursor, the cursor returns to where it was before the change", false, EMUCMD_TASEDITOR_SWITCH_AUTORESTORING, 0,
+	WINDOWITEMS_RECORDER_BOX, IDC_RECORDER_BOX, -1, 0, 0, 0, "", false, 0, 0,
+	WINDOWITEMS_RECORDING, IDC_RECORDING, -1, 0, 0, 0, "Switch Input Recording on/off", false, EMUCMD_MOVIE_READONLY_TOGGLE, 0,
+	WINDOWITEMS_RECORD_ALL, IDC_RADIO_ALL, -1, 0, 0, 0, "Switch off Multitracking", false, 0, 0,
+	WINDOWITEMS_RECORD_1P, IDC_RADIO_1P, -1, 0, 0, 0, "Select Joypad 1 as current", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
+	WINDOWITEMS_RECORD_2P, IDC_RADIO_2P, -1, 0, 0, 0, "Select Joypad 2 as current", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
+	WINDOWITEMS_RECORD_3P, IDC_RADIO_3P, -1, 0, 0, 0, "Select Joypad 3 as current", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
+	WINDOWITEMS_RECORD_4P, IDC_RADIO_4P, -1, 0, 0, 0, "Select Joypad 4 as current", false, EMUCMD_TASEDITOR_SWITCH_MULTITRACKING, 0,
+	WINDOWITEMS_SUPERIMPOSE, IDC_SUPERIMPOSE, -1, 0, 0, 0, "Allows to superimpose old Input with new buttons, instead of overwriting", false, 0, 0,
+	WINDOWITEMS_USE_PATTERN, IDC_USEPATTERN, -1, 0, 0, 0, "Applies current Autofire Pattern to Input recording", false, 0, 0,
+	WINDOWITEMS_SPLICER_BOX, IDC_SPLICER_BOX, -1, 0, 0, 0, "", false, 0, 0,
+	WINDOWITEMS_SELECTION_TEXT, IDC_TEXT_SELECTION, -1, 0, 0, 0, "Current size of Selection", false, 0, 0,
+	WINDOWITEMS_CLIPBOARD_TEXT, IDC_TEXT_CLIPBOARD, -1, 0, 0, 0, "Current size of Input in the Clipboard", false, 0, 0,
+	WINDOWITEMS_LUA_BOX, IDC_LUA_BOX, -1, 0, 0, 0, "", false, 0, 0,
+	WINDOWITEMS_RUN_MANUAL, TASEDITOR_RUN_MANUAL, -1, 0, 0, 0, "Press the button to execute Lua Manual Function", false, EMUCMD_TASEDITOR_RUN_MANUAL_LUA, 0,
+	WINDOWITEMS_RUN_AUTO, IDC_RUN_AUTO, -1, 0, 0, 0, "Enable Lua Auto Function (but first it must be registered by Lua script)", false, 0, 0,
+	WINDOWITEMS_BRANCHES_BUTTON, IDC_BRANCHES_BUTTON, -1, 0, 0, 0, "Click here to switch between Bookmarks List and Branches Tree", false, 0, 0,
+	WINDOWITEMS_BOOKMARKS_BOX, IDC_BOOKMARKS_BOX, -1, 0, 0, 0, "", false, 0, 0,
+	WINDOWITEMS_BOOKMARKS_LIST, IDC_BOOKMARKSLIST, -1, 0, 0, 0, "Right click = set Bookmark, Left click = jump to Bookmark or load Branch", false, 0, 0,
+	WINDOWITEMS_BRANCHES_BITMAP, IDC_BRANCHES_BITMAP, -1, 0, 0, 0, "Right click = set Bookmark, single Left click = jump to Bookmark, double Left click = load Branch", false, 0, 0,
+	WINDOWITEMS_HISTORY_BOX, IDC_HISTORY_BOX, -1, 0, 0, -1, "", false, 0, 0,
+	WINDOWITEMS_HISTORY_LIST, IDC_HISTORYLIST, -1, 0, 0, -1, "Click to revert the project back to that time", false, 0, 0,
+	WINDOWITEMS_PREVIOUS_MARKER, TASEDITOR_PREV_MARKER, -1, -1, 0, -1, "Send Selection to previous Marker (mouse: Ctrl+Wheel up) (hotkey: Ctrl+PageUp)", false, 0, 0,
+	WINDOWITEMS_SIMILAR, TASEDITOR_FIND_BEST_SIMILAR_MARKER, -1, -1, 0, -1, "Auto-search for Marker Note", false, 0, 0,
+	WINDOWITEMS_MORE, TASEDITOR_FIND_NEXT_SIMILAR_MARKER, -1, -1, 0, -1, "Continue Auto-search", false, 0, 0,
+	WINDOWITEMS_NEXT_MARKER, TASEDITOR_NEXT_MARKER, -1, -1, 0, -1, "Send Selection to next Marker (mouse: Ctrl+Wheel up) (hotkey: Ctrl+PageDown)", false, 0, 0,
 };
 
 TASEDITOR_WINDOW::TASEDITOR_WINDOW()
@@ -186,6 +186,66 @@ TASEDITOR_WINDOW::TASEDITOR_WINDOW()
 	minHeight = 0;
 }
 
+void TASEDITOR_WINDOW::updateTooltips()
+{
+	for (int i = 0; i < TASEDITOR_WINDOW_TOTAL_ITEMS; ++i)
+	{
+		TOOLINFO toolInfo = { 0 };
+		toolInfo.cbSize = sizeof(TOOLINFO);
+		toolInfo.hwnd = hwndTASEditor;
+		toolInfo.uId = (UINT_PTR)GetDlgItem(hwndTASEditor, windowItems[i].id);
+		if (windowItems[i].isStaticRect)
+		{
+			// for static text we specify rectangle
+			RECT toolRect;
+			GetWindowRect(GetDlgItem(hwndTASEditor, windowItems[i].id), &toolRect);
+			POINT pt;
+			pt.x = toolRect.left;
+			pt.y = toolRect.top;
+			ScreenToClient(hwndTASEditor, &pt);
+			toolInfo.rect.left = pt.x;
+			toolInfo.rect.top = pt.y;
+			toolInfo.rect.right = toolInfo.rect.left + toolRect.right - toolRect.left;
+			toolInfo.rect.bottom = toolInfo.rect.top + toolRect.bottom - toolRect.top;
+			toolInfo.uFlags = TTF_SUBCLASS;
+		}
+		else
+			// for other controls we provide hwnd
+			toolInfo.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
+		char tooltipText[TOOLTIP_TEXT_MAX_LEN];
+		if (windowItems[i].hotkeyEmuCmd && FCEUD_CommandMapping[windowItems[i].hotkeyEmuCmd].NumC)
+		{
+			// add hotkey mapping if needed
+			strcpy(tooltipText, windowItems[i].tooltipText);
+			strcat(tooltipText, " (hotkey: ");
+			char *buttName = MakeButtString(&FCEUD_CommandMapping[windowItems[i].hotkeyEmuCmd], 0);
+			strcat(tooltipText, buttName);
+			free(buttName);
+			strcat(tooltipText, ")");
+			toolInfo.lpszText = tooltipText;
+		} else
+			// No hotkey, use the tooltip text itself
+			toolInfo.lpszText = windowItems[i].tooltipText;
+
+		if(!windowItems[i].tooltipHWND)
+		{
+			// if the tooltip handle is not created yet, create one
+			if (windowItems[i].tooltipHWND = CreateWindowEx(NULL, TOOLTIPS_CLASS, NULL,
+				WS_POPUP | TTS_ALWAYSTIP | TTS_BALLOON | TTS_NOANIMATE | TTS_NOFADE,
+				CW_USEDEFAULT, CW_USEDEFAULT,
+				CW_USEDEFAULT, CW_USEDEFAULT,
+				hwndTASEditor, NULL,
+				fceu_hInstance, NULL))
+			{
+				SendMessage(windowItems[i].tooltipHWND, TTM_ADDTOOL, 0, (LPARAM)&toolInfo);
+				SendMessage(windowItems[i].tooltipHWND, TTM_SETDELAYTIME, TTDT_AUTOPOP, TOOLTIPS_AUTOPOP_TIMEOUT);
+			}
+		} else
+			// The tooltip handle is already created, just update the tooltip info
+			SendMessage(windowItems[i].tooltipHWND, TTM_SETTOOLINFO, 0, (LPARAM)&toolInfo);
+	}
+}
+
 void TASEDITOR_WINDOW::init()
 {
 	isReadyForResizing = false;
@@ -203,89 +263,36 @@ void TASEDITOR_WINDOW::init()
 	updateCheckedItems();
 	hPatternsMenu = GetSubMenu(hMainMenu, PATTERNS_MENU_POS);
 	// tooltips
-	for (int i = 0; i < TASEDITOR_WINDOW_TOTAL_ITEMS; ++i)
-	{
-		if (windowItems[i].tooltipTextBase[0])
-		{
-			windowItems[i].tooltipHWND = CreateWindowEx(NULL, TOOLTIPS_CLASS, NULL,
-									  WS_POPUP | TTS_ALWAYSTIP | TTS_BALLOON | TTS_NOANIMATE | TTS_NOFADE,
-									  CW_USEDEFAULT, CW_USEDEFAULT,
-									  CW_USEDEFAULT, CW_USEDEFAULT,
-									  hwndTASEditor, NULL, 
-									  fceu_hInstance, NULL);
-			if (windowItems[i].tooltipHWND)
-			{
-				// Associate the tooltip with the tool
-				TOOLINFO toolInfo = {0};
-				toolInfo.cbSize = sizeof(toolInfo);
-				toolInfo.hwnd = hwndTASEditor;
-				toolInfo.uId = (UINT_PTR)GetDlgItem(hwndTASEditor, windowItems[i].id);
-				if (windowItems[i].isStaticRect)
-				{
-					// for static text we specify rectangle
-					toolInfo.uFlags = TTF_SUBCLASS;
-					RECT toolRect;
-					GetWindowRect(GetDlgItem(hwndTASEditor, windowItems[i].id), &toolRect);
-					POINT pt;
-					pt.x = toolRect.left;
-					pt.y = toolRect.top;
-					ScreenToClient(hwndTASEditor, &pt);
-					toolInfo.rect.left = pt.x;
-					toolInfo.rect.right = toolInfo.rect.left + (toolRect.right - toolRect.left);
-					toolInfo.rect.top = pt.y;
-					toolInfo.rect.bottom = toolInfo.rect.top + (toolRect.bottom - toolRect.top);
-				} else
-				{
-					// for other controls we provide hwnd
-					toolInfo.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
-				}
-				// add hotkey mapping if needed
-				if (windowItems[i].hotkeyEmuCmd && FCEUD_CommandMapping[windowItems[i].hotkeyEmuCmd])
-				{
-					windowItems[i].tooltipText[0] = 0;
-					strcpy(windowItems[i].tooltipText, windowItems[i].tooltipTextBase);
-					strcat(windowItems[i].tooltipText, " (hotkey: ");
-					strncat(windowItems[i].tooltipText, GetKeyComboName(FCEUD_CommandMapping[windowItems[i].hotkeyEmuCmd]), TOOLTIP_TEXT_MAX_LEN - strlen(windowItems[i].tooltipText) - 1);
-					strncat(windowItems[i].tooltipText, ")", TOOLTIP_TEXT_MAX_LEN - strlen(windowItems[i].tooltipText) - 1);
-					toolInfo.lpszText = windowItems[i].tooltipText;
-				} else
-				{
-					toolInfo.lpszText = windowItems[i].tooltipTextBase;
-				}
-				SendMessage(windowItems[i].tooltipHWND, TTM_ADDTOOL, 0, (LPARAM)&toolInfo);
-				SendMessage(windowItems[i].tooltipHWND, TTM_SETDELAYTIME, TTDT_AUTOPOP, TOOLTIPS_AUTOPOP_TIMEOUT);
-			}
-		}
-	}
 	updateTooltips();
+	toggleTooltips();
 	// subclass "Marker X" text fields
-	IDC_PLAYBACK_MARKER_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_PLAYBACK_MARKER), GWL_WNDPROC, (LONG)IDC_PLAYBACK_MARKER_WndProc);
-	IDC_SELECTION_MARKER_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_SELECTION_MARKER), GWL_WNDPROC, (LONG)IDC_SELECTION_MARKER_WndProc);
+	IDC_PLAYBACK_MARKER_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_PLAYBACK_MARKER), GWLP_WNDPROC, (LONG_PTR)IDC_PLAYBACK_MARKER_WndProc);
+	IDC_SELECTION_MARKER_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_SELECTION_MARKER), GWLP_WNDPROC, (LONG_PTR)IDC_SELECTION_MARKER_WndProc);
 	// subclass all buttons
-	IDC_PROGRESS_BUTTON_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_PROGRESS_BUTTON), GWL_WNDPROC, (LONG)IDC_PROGRESS_BUTTON_WndProc);
-	IDC_BRANCHES_BUTTON_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_BRANCHES_BUTTON), GWL_WNDPROC, (LONG)IDC_BRANCHES_BUTTON_WndProc);
-	TASEDITOR_REWIND_FULL_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_REWIND_FULL), GWL_WNDPROC, (LONG)TASEDITOR_REWIND_FULL_WndProc);
-	TASEDITOR_REWIND_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_REWIND), GWL_WNDPROC, (LONG)TASEDITOR_REWIND_WndProc);
-	TASEDITOR_PLAYSTOP_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_PLAYSTOP), GWL_WNDPROC, (LONG)TASEDITOR_PLAYSTOP_WndProc);
-	TASEDITOR_FORWARD_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_FORWARD), GWL_WNDPROC, (LONG)TASEDITOR_FORWARD_WndProc);
-	TASEDITOR_FORWARD_FULL_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_FORWARD_FULL), GWL_WNDPROC, (LONG)TASEDITOR_FORWARD_FULL_WndProc);
-	CHECK_FOLLOW_CURSOR_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, CHECK_FOLLOW_CURSOR), GWL_WNDPROC, (LONG)CHECK_FOLLOW_CURSOR_WndProc);
-	CHECK_AUTORESTORE_PLAYBACK_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, CHECK_AUTORESTORE_PLAYBACK), GWL_WNDPROC, (LONG)CHECK_AUTORESTORE_PLAYBACK_WndProc);
-	IDC_RADIO_ALL_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RADIO_ALL), GWL_WNDPROC, (LONG)IDC_RADIO_ALL_WndProc);
-	IDC_RADIO_1P_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RADIO_1P), GWL_WNDPROC, (LONG)IDC_RADIO_1P_WndProc);
-	IDC_RADIO_2P_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RADIO_2P), GWL_WNDPROC, (LONG)IDC_RADIO_2P_WndProc);
-	IDC_RADIO_3P_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RADIO_3P), GWL_WNDPROC, (LONG)IDC_RADIO_3P_WndProc);
-	IDC_RADIO_4P_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RADIO_4P), GWL_WNDPROC, (LONG)IDC_RADIO_4P_WndProc);
-	IDC_SUPERIMPOSE_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_SUPERIMPOSE), GWL_WNDPROC, (LONG)IDC_SUPERIMPOSE_WndProc);
-	IDC_USEPATTERN_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_USEPATTERN), GWL_WNDPROC, (LONG)IDC_USEPATTERN_WndProc);
-	TASEDITOR_PREV_MARKER_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_PREV_MARKER), GWL_WNDPROC, (LONG)TASEDITOR_PREV_MARKER_WndProc);
-	TASEDITOR_FIND_BEST_SIMILAR_MARKER_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_FIND_BEST_SIMILAR_MARKER), GWL_WNDPROC, (LONG)TASEDITOR_FIND_BEST_SIMILAR_MARKER_WndProc);
-	TASEDITOR_FIND_NEXT_SIMILAR_MARKER_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_FIND_NEXT_SIMILAR_MARKER), GWL_WNDPROC, (LONG)TASEDITOR_FIND_NEXT_SIMILAR_MARKER_WndProc);
-	TASEDITOR_NEXT_MARKER_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_NEXT_MARKER), GWL_WNDPROC, (LONG)TASEDITOR_NEXT_MARKER_WndProc);
-	CHECK_TURBO_SEEK_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, CHECK_TURBO_SEEK), GWL_WNDPROC, (LONG)CHECK_TURBO_SEEK_WndProc);
-	IDC_RECORDING_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RECORDING), GWL_WNDPROC, (LONG)IDC_RECORDING_WndProc);
-	TASEDITOR_RUN_MANUAL_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, TASEDITOR_RUN_MANUAL), GWL_WNDPROC, (LONG)TASEDITOR_RUN_MANUAL_WndProc);
-	IDC_RUN_AUTO_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndTASEditor, IDC_RUN_AUTO), GWL_WNDPROC, (LONG)IDC_RUN_AUTO_WndProc);
+	IDC_PROGRESS_BUTTON_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_PROGRESS_BUTTON), GWLP_WNDPROC, (LONG_PTR)IDC_PROGRESS_BUTTON_WndProc);
+	IDC_BRANCHES_BUTTON_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_BRANCHES_BUTTON), GWLP_WNDPROC, (LONG_PTR)IDC_BRANCHES_BUTTON_WndProc);
+	TASEDITOR_REWIND_FULL_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_REWIND_FULL), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_REWIND_FULL_WndProc);
+	TASEDITOR_REWIND_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_REWIND), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_REWIND_WndProc);
+	TASEDITOR_PLAYSTOP_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_PLAYSTOP), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_PLAYSTOP_WndProc);
+	TASEDITOR_FORWARD_oldWndProc =(WNDPROC) SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_FORWARD), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_FORWARD_WndProc);
+	TASEDITOR_FORWARD_FULL_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_FORWARD_FULL), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_FORWARD_FULL_WndProc);
+	CHECK_FOLLOW_CURSOR_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, CHECK_FOLLOW_CURSOR), GWLP_WNDPROC, (LONG_PTR)CHECK_FOLLOW_CURSOR_WndProc);
+	CHECK_AUTORESTORE_PLAYBACK_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, CHECK_AUTORESTORE_PLAYBACK), GWLP_WNDPROC, (LONG_PTR)CHECK_AUTORESTORE_PLAYBACK_WndProc);
+	IDC_RADIO_ALL_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RADIO_ALL), GWLP_WNDPROC, (LONG_PTR)IDC_RADIO_ALL_WndProc);
+	IDC_RADIO_1P_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RADIO_1P), GWLP_WNDPROC, (LONG_PTR)IDC_RADIO_1P_WndProc);
+	IDC_RADIO_2P_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RADIO_2P), GWLP_WNDPROC, (LONG_PTR)IDC_RADIO_2P_WndProc);
+	IDC_RADIO_3P_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RADIO_3P), GWLP_WNDPROC, (LONG_PTR)IDC_RADIO_3P_WndProc);
+	IDC_RADIO_4P_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RADIO_4P), GWLP_WNDPROC, (LONG_PTR)IDC_RADIO_4P_WndProc);
+	IDC_SUPERIMPOSE_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_SUPERIMPOSE), GWLP_WNDPROC, (LONG_PTR)IDC_SUPERIMPOSE_WndProc);
+	IDC_USEPATTERN_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_USEPATTERN), GWLP_WNDPROC, (LONG_PTR)IDC_USEPATTERN_WndProc);
+	TASEDITOR_PREV_MARKER_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_PREV_MARKER), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_PREV_MARKER_WndProc);
+	TASEDITOR_FIND_BEST_SIMILAR_MARKER_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_FIND_BEST_SIMILAR_MARKER), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_FIND_BEST_SIMILAR_MARKER_WndProc);
+	TASEDITOR_FIND_NEXT_SIMILAR_MARKER_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_FIND_NEXT_SIMILAR_MARKER), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_FIND_NEXT_SIMILAR_MARKER_WndProc);
+	TASEDITOR_NEXT_MARKER_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_NEXT_MARKER), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_NEXT_MARKER_WndProc);
+	CHECK_TURBO_SEEK_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, CHECK_TURBO_SEEK), GWLP_WNDPROC, (LONG_PTR)CHECK_TURBO_SEEK_WndProc);
+	IDC_RECORDING_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RECORDING), GWLP_WNDPROC, (LONG_PTR)IDC_RECORDING_WndProc);
+	TASEDITOR_RUN_MANUAL_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, TASEDITOR_RUN_MANUAL), GWLP_WNDPROC, (LONG_PTR)TASEDITOR_RUN_MANUAL_WndProc);
+	IDC_RUN_AUTO_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndTASEditor, IDC_RUN_AUTO), GWLP_WNDPROC, (LONG_PTR)IDC_RUN_AUTO_WndProc);
 	// create "Recent" submenu
 	hRecentProjectsMenu = CreateMenu();
 	updateRecentProjectsMenu();
@@ -535,7 +542,7 @@ void TASEDITOR_WINDOW::changeBookmarksListHeight(int newHeight)
 		ShowWindow(hwndTASEditor, SW_SHOWMAXIMIZED);
 }
 
-void TASEDITOR_WINDOW::updateTooltips()
+void TASEDITOR_WINDOW::toggleTooltips()
 {
 	if (taseditorConfig.tooltipsEnabled)
 	{
@@ -775,7 +782,7 @@ void TASEDITOR_WINDOW::recheckPatternsMenu()
 }
 
 // ====================================================================================================
-BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	extern TASEDITOR_WINDOW taseditorWindow;
 	switch(uMsg)
@@ -828,7 +835,7 @@ BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPara
 				switch(((LPNMHDR)lParam)->code)
 				{
 				case NM_CUSTOMDRAW:
-					SetWindowLong(hWnd, DWL_MSGRESULT, pianoRoll.handleCustomDraw((NMLVCUSTOMDRAW*)lParam));
+					SetWindowLongPtr(hWnd, DWLP_MSGRESULT, pianoRoll.handleCustomDraw((NMLVCUSTOMDRAW*)lParam));
 					return TRUE;
 				case LVN_GETDISPINFO:
 					pianoRoll.getDispInfo((NMLVDISPINFO*)lParam);
@@ -849,7 +856,7 @@ BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPara
 				switch(((LPNMHDR)lParam)->code)
 				{
 				case NM_CUSTOMDRAW:
-					SetWindowLong(hWnd, DWL_MSGRESULT, bookmarks.handleCustomDraw((NMLVCUSTOMDRAW*)lParam));
+					SetWindowLongPtr(hWnd, DWLP_MSGRESULT, bookmarks.handleCustomDraw((NMLVCUSTOMDRAW*)lParam));
 					return TRUE;
 				case LVN_GETDISPINFO:
 					bookmarks.getDispInfo((NMLVDISPINFO*)lParam);
@@ -860,7 +867,7 @@ BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPara
 				switch(((LPNMHDR)lParam)->code)
 				{
 				case NM_CUSTOMDRAW:
-					SetWindowLong(hWnd, DWL_MSGRESULT, history.handleCustomDraw((NMLVCUSTOMDRAW*)lParam));
+					SetWindowLongPtr(hWnd, DWLP_MSGRESULT, history.handleCustomDraw((NMLVCUSTOMDRAW*)lParam));
 					return TRUE;
 				case LVN_GETDISPINFO:
 					history.getDispInfo((NMLVDISPINFO*)lParam);
@@ -890,12 +897,12 @@ BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPara
 			{
 				SetTextColor((HDC)wParam, PLAYBACK_MARKER_COLOR);
 				SetBkMode((HDC)wParam, TRANSPARENT);
-				return (BOOL)(pianoRoll.bgBrush);
+				return (INT_PTR)(pianoRoll.bgBrush);
 			} else if ((HWND)lParam == selection.hwndSelectionMarkerNumber)
 			{
 				SetTextColor((HDC)wParam, GetSysColor(COLOR_HIGHLIGHT));
 				SetBkMode((HDC)wParam, TRANSPARENT);
-				return (BOOL)pianoRoll.bgBrush;
+				return (INT_PTR)pianoRoll.bgBrush;
 			}
 			break;
 		case WM_COMMAND:
@@ -1133,19 +1140,19 @@ BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPara
 					FCEUI_MovieToggleReadOnly();
 					CheckDlgButton(taseditorWindow.hwndTASEditor, IDC_RECORDING, movie_readonly?BST_UNCHECKED : BST_CHECKED);
 					break;
-				case IDC_RADIO2:
+				case IDC_RADIO_ALL:
 					recorder.multitrackRecordingJoypadNumber = MULTITRACK_RECORDING_ALL;
 					break;
-				case IDC_RADIO3:
+				case IDC_RADIO_1P:
 					recorder.multitrackRecordingJoypadNumber = MULTITRACK_RECORDING_1P;
 					break;
-				case IDC_RADIO4:
+				case IDC_RADIO_2P:
 					recorder.multitrackRecordingJoypadNumber = MULTITRACK_RECORDING_2P;
 					break;
-				case IDC_RADIO5:
+				case IDC_RADIO_3P:
 					recorder.multitrackRecordingJoypadNumber = MULTITRACK_RECORDING_3P;
 					break;
-				case IDC_RADIO6:
+				case IDC_RADIO_4P:
 					recorder.multitrackRecordingJoypadNumber = MULTITRACK_RECORDING_4P;
 					break;
 				case IDC_SUPERIMPOSE:
@@ -1258,7 +1265,7 @@ BOOL CALLBACK TASEditorWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPara
 				case ID_HELP_TOOLTIPS:
 					taseditorConfig.tooltipsEnabled ^= 1;
 					taseditorWindow.updateCheckedItems();
-					taseditorWindow.updateTooltips();
+					taseditorWindow.toggleTooltips();
 					break;
 				case ID_HELP_ABOUT:
 					DialogBox(fceu_hInstance, MAKEINTRESOURCE(IDD_TASEDITOR_ABOUT), taseditorWindow.hwndTASEditor, aboutWndProc);
