@@ -19,45 +19,47 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include        "types.h"
-#include        "x6502.h"
-#include        "fceu.h"
-#include        "ppu.h"
-#include        "nsf.h"
-#include        "sound.h"
-#include        "file.h"
-#include        "utils/endian.h"
-#include        "utils/memory.h"
+#include "types.h"
+#include "x6502.h"
+#include "fceu.h"
+#include "ppu.h"
+#include "nsf.h"
+#include "sound.h"
+#include "file.h"
+#include "utils/endian.h"
+#include "utils/memory.h"
+		 
+#include "cart.h"
+#include "palette.h"
+#include "state.h"
+#include "video.h"
+#include "input.h"
+#include "driver.h"
+#include "debug.h"
+		 
+#include <cstring>
+#include <cstdio>
+#include <cstdlib>
 
-#include        "cart.h"
-#include        "palette.h"
-#include        "state.h"
-#include        "video.h"
-#include        "input.h"
-#include        "driver.h"
-#include        "debug.h"
+#define VBlankON    (PPU[0] & 0x80)	//Generate VBlank NMI
+#define Sprite16    (PPU[0] & 0x20)	//Sprites 8x16/8x8
+#define BGAdrHI     (PPU[0] & 0x10)	//BG pattern adr $0000/$1000
+#define SpAdrHI     (PPU[0] & 0x08)	//Sprite pattern adr $0000/$1000
+#define INC32       (PPU[0] & 0x04)	//auto increment 1/32
 
-#include        <cstring>
-#include        <cstdio>
-#include        <cstdlib>
+#define SpriteON    (PPU[1] & 0x10)	//Show Sprite
+#define ScreenON    (PPU[1] & 0x08)	//Show screen
+#define PPUON       (PPU[1] & 0x18)	//PPU should operate
+#define GRAYSCALE   (PPU[1] & 0x01)	//Grayscale (AND palette entries with 0x30)
 
-#define VBlankON        (PPU[0] & 0x80)		//Generate VBlank NMI
-#define Sprite16        (PPU[0] & 0x20)		//Sprites 8x16/8x8
-#define BGAdrHI         (PPU[0] & 0x10)		//BG pattern adr $0000/$1000
-#define SpAdrHI         (PPU[0] & 0x08)		//Sprite pattern adr $0000/$1000
-#define INC32           (PPU[0] & 0x04)		//auto increment 1/32
+#define SpriteLeft8 (PPU[1] & 0x04)
+#define BGLeft8     (PPU[1] & 0x02)
 
-#define SpriteON        (PPU[1] & 0x10)		//Show Sprite
-#define ScreenON        (PPU[1] & 0x08)		//Show screen
-#define PPUON           (PPU[1] & 0x18)		//PPU should operate
-#define GRAYSCALE       (PPU[1] & 0x01)		//Grayscale (AND palette entries with 0x30)
+#define PPU_status  (PPU[2])
 
-#define SpriteLeft8     (PPU[1] & 0x04)
-#define BGLeft8         (PPU[1] & 0x02)
-
-#define PPU_status      (PPU[2])
-
-#define Pal             (PALRAM)
+#define READPALNOGS(ofs)    (PALRAM[(ofs)])
+#define READPAL(ofs)    (PALRAM[(ofs)] & (GRAYSCALE ? 0x30 : 0xFF))
+#define READUPAL(ofs)   (UPALRAM[(ofs)] & (GRAYSCALE ? 0x30 : 0xFF))
 
 static void FetchSpriteData(void);
 static void RefreshLine(int lastpixel);
@@ -69,6 +71,8 @@ static uint32 ppulut1[256];
 static uint32 ppulut2[256];
 static uint32 ppulut3[128];
 
+static bool new_ppu_reset = false;
+
 int test = 0;
 
 template<typename T, int BITS>
@@ -274,8 +278,24 @@ struct PPUREGS {
 		v &= 1;
 		fv &= 7;
 	}
+
+	void debug_log()
+	{
+		FCEU_printf("ppur: fv(%d), v(%d), h(%d), vt(%d), ht(%d)\n",fv,v,h,vt,ht);
+		FCEU_printf("      _fv(%d), _v(%d), _h(%d), _vt(%d), _ht(%d)\n",_fv,_v,_h,_vt,_ht);
+		FCEU_printf("      fh(%d), s(%d), par(%d)\n",fh,s,par);
+		FCEU_printf("      .status cycle(%d), end_cycle(%d), sl(%d)\n",status.cycle,status.end_cycle,status.sl);
+	}
 } ppur;
 
+int newppu_get_scanline() { return ppur.status.sl; }
+int newppu_get_dot() { return ppur.status.cycle; }
+void newppu_hacky_emergency_reset()
+{
+	if(ppur.status.end_cycle == 0)
+		ppur.reset();
+}
+
 static void makeppulut(void) {
 	int x;
 	int y;
@@ -311,7 +331,7 @@ int fceuindbg = 0;
 //0xFF shall indicate to use palette[0]
 uint8 gNoBGFillColor = 0xFF;
 
-int MMC5Hack = 0, PEC586Hack = 0;;
+int MMC5Hack = 0;
 uint32 MMC5HackVROMMask = 0;
 uint8 *MMC5HackExNTARAMPtr = 0;
 uint8 *MMC5HackVROMPTR = 0;
@@ -321,6 +341,12 @@ uint8 MMC50x5130 = 0;
 uint8 MMC5HackSPScroll = 0;
 uint8 MMC5HackSPPage = 0;
 
+int PEC586Hack = 0;
+
+int QTAIHack = 0;
+uint8 QTAINTRAM[2048];
+uint8 qtaintramreg;
+
 uint8 VRAMBuffer = 0, PPUGenLatch = 0;
 uint8 *vnapage[4];
 uint8 PPUNTARAM = 0;
@@ -335,8 +361,9 @@ void (*PPU_hook)(uint32 A);
 
 uint8 vtoggle = 0;
 uint8 XOffset = 0;
+uint8 SpriteDMA = 0; // $4014 / Writing $xx copies 256 bytes by reading from $xx00-$xxFF and writing to $2004 (OAM data)
 
-uint32 TempAddr = 0, RefreshAddr = 0, DummyRead = 0;
+uint32 TempAddr = 0, RefreshAddr = 0, DummyRead = 0, NTRefreshAddr = 0;
 
 static int maxsprites = 8;
 
@@ -354,17 +381,18 @@ uint8 UPALRAM[0x03];//for 0x4/0x8/0xC addresses in palette, the ones in
 #define MMC5SPRVRAMADR(V)   &MMC5SPRVPage[(V) >> 10][(V)]
 #define VRAMADR(V)          &VPage[(V) >> 10][(V)]
 
-//mbg 8/6/08 - fix a bug relating to
-//"When in 8x8 sprite mode, only one set is used for both BG and sprites."
-//in mmc5 docs
-uint8 * MMC5BGVRAMADR(uint32 V) {
-	if (!Sprite16) {
-		extern uint8 mmc5ABMode;				/* A=0, B=1 */
-		if (mmc5ABMode == 0)
-			return MMC5SPRVRAMADR(V);
+uint8* MMC5BGVRAMADR(uint32 A);
+
+uint8 READPAL_MOTHEROFALL(uint32 A)
+{
+	if(!(A & 3)) {
+		if(!(A & 0xC))
+			return READPAL(0x00);
 		else
-			return &MMC5BGVPage[(V) >> 10][(V)];
-	} else return &MMC5BGVPage[(V) >> 10][(V)];
+			return READUPAL(((A & 0xC) >> 2) - 1);
+	}
+	else
+		return READPAL(A & 0x1F);
 }
 
 //this duplicates logic which is embedded in the ppu rendering code
@@ -405,12 +433,18 @@ inline void FFCEUX_PPUWrite_Default(uint32 A, uint8 V) {
 		if (PPUCHRRAM & (1 << (tmp >> 10)))
 			VPage[tmp >> 10][tmp] = V;
 	} else if (tmp < 0x3F00) {
-		if (PPUNTARAM & (1 << ((tmp & 0xF00) >> 10)))
-			vnapage[((tmp & 0xF00) >> 10)][tmp & 0x3FF] = V;
+		if (QTAIHack && (qtaintramreg & 1)) {
+			QTAINTRAM[((((tmp & 0xF00) >> 10) >> ((qtaintramreg >> 1)) & 1) << 10) | (tmp & 0x3FF)] = V;
+		} else {
+			if (PPUNTARAM & (1 << ((tmp & 0xF00) >> 10)))
+				vnapage[((tmp & 0xF00) >> 10)][tmp & 0x3FF] = V;
+		}
 	} else {
 		if (!(tmp & 3)) {
-			if (!(tmp & 0xC))
+			if (!(tmp & 0xC)) {
 				PALRAM[0x00] = PALRAM[0x04] = PALRAM[0x08] = PALRAM[0x0C] = V & 0x3F;
+				PALRAM[0x10] = PALRAM[0x14] = PALRAM[0x18] = PALRAM[0x1C] = V & 0x3F;
+			}
 			else
 				UPALRAM[((tmp & 0xC) >> 2) - 1] = V & 0x3F;
 		} else
@@ -419,13 +453,37 @@ inline void FFCEUX_PPUWrite_Default(uint32 A, uint8 V) {
 }
 
 volatile int rendercount, vromreadcount, undefinedvromcount, LogAddress = -1;
-unsigned char *cdloggervdata;
+unsigned char *cdloggervdata = NULL;
 unsigned int cdloggerVideoDataSize = 0;
 
-int GetCHRAddress(int A) {
-	if (cdloggerVideoDataSize) {
-		int result = &VPage[A >> 10][A] - CHRptr[0];
+int GetCHRAddress(int A) 
+{
+	if (cdloggerVideoDataSize) 
+	{
+		int result = -1;
+		if ( (A >= 0) && (A < 0x2000) )
+		{
+			result = &VPage[A >> 10][A] - CHRptr[0];
+		}
 		if ((result >= 0) && (result < (int)cdloggerVideoDataSize))
+		{
+			return result;
+		}
+	}
+	else
+	{
+		if ( (A >= 0) && (A < 0x2000) ) return A;
+	}
+	return -1;
+}
+
+int GetCHROffset(uint8 *ptr) {
+	int result = ptr - CHRptr[0];
+	if (cdloggerVideoDataSize) {
+		if ((result >= 0) && (result < (int)cdloggerVideoDataSize))
+			return result;
+	} else {
+		if ((result >= 0) && (result < 0x2000))
 			return result;
 	}
 	return -1;
@@ -440,8 +498,28 @@ int GetCHRAddress(int A) {
 				if (!(cdloggervdata[addr] & 1))	\
 				{ \
 					cdloggervdata[addr] |= 1; \
-					if (!(cdloggervdata[addr] & 2)) undefinedvromcount--; \
-					rendercount++; \
+					if(cdloggerVideoDataSize) { \
+						if (!(cdloggervdata[addr] & 2)) undefinedvromcount--; \
+						rendercount++; \
+					} \
+				} \
+			} \
+		} \
+}
+
+#define RENDER_LOGP(tmp) { \
+		if (debug_loggingCD) \
+		{ \
+			int addr = GetCHROffset(tmp); \
+			if (addr != -1)	\
+			{ \
+				if (!(cdloggervdata[addr] & 1))	\
+				{ \
+					cdloggervdata[addr] |= 1; \
+					if(cdloggerVideoDataSize) { \
+						if (!(cdloggervdata[addr] & 2)) undefinedvromcount--; \
+						rendercount++; \
+					} \
 				} \
 			} \
 		} \
@@ -460,14 +538,11 @@ uint8 FASTCALL FFCEUX_PPURead_Default(uint32 A) {
 		uint8 ret;
 		if (!(tmp & 3)) {
 			if (!(tmp & 0xC))
-				ret = PALRAM[0x00];
+				ret = READPAL(0x00);
 			else
-				ret = UPALRAM[((tmp & 0xC) >> 2) - 1];
+				ret = READUPAL(((tmp & 0xC) >> 2) - 1);
 		} else
-			ret = PALRAM[tmp & 0x1F];
-
-		if (GRAYSCALE)
-			ret &= 0x30;
+			ret = READPAL(tmp & 0x1F);
 		return ret;
 	}
 }
@@ -480,7 +555,7 @@ void (*FFCEUX_PPUWrite)(uint32 A, uint8 V) = 0;
 
 #define CALL_PPUWRITE(A, V) (FFCEUX_PPUWrite ? FFCEUX_PPUWrite(A, V) : FFCEUX_PPUWrite_Default(A, V))
 
-//whether to use the new ppu (new PPU doesn't handle MMC5 extra nametables at all
+//whether to use the new ppu
 int newppu = 0;
 
 void ppu_getScroll(int &xpos, int &ypos) {
@@ -664,7 +739,7 @@ static DECLFR(A2007) {
 		if (!DummyRead && (LogAddress != -1)) {
 			if (!(cdloggervdata[LogAddress] & 2)) {
 				cdloggervdata[LogAddress] |= 2;
-				if (!(cdloggervdata[LogAddress] & 1)) undefinedvromcount--;
+				if ((!(cdloggervdata[LogAddress] & 1)) && cdloggerVideoDataSize) undefinedvromcount--;
 				vromreadcount++;
 			}
 		} else
@@ -682,13 +757,11 @@ static DECLFR(A2007) {
 			//to get a gray color reading
 			if (!(tmp & 3)) {
 				if (!(tmp & 0xC))
-					ret = PALRAM[0x00];
+					ret = READPAL(0x00);
 				else
-					ret = UPALRAM[((tmp & 0xC) >> 2) - 1];
+					ret = READUPAL(((tmp & 0xC) >> 2) - 1);
 			} else
-				ret = PALRAM[tmp & 0x1F];
-			if (GRAYSCALE)
-				ret &= 0x30;
+				ret = READPAL(tmp & 0x1F);
 			VRAMBuffer = CALL_PPUREAD(RefreshAddr - 0x1000);
 		} else {
 			if (debug_loggingCD && (RefreshAddr < 0x2000))
@@ -699,18 +772,18 @@ static DECLFR(A2007) {
 		RefreshAddr = ppur.get_2007access();
 		return ret;
 	} else {
+
+		//OLDPPU
 		FCEUPPU_LineUpdate();
 
 		if (tmp >= 0x3F00) {	// Palette RAM tied directly to the output data, without VRAM buffer
 			if (!(tmp & 3)) {
 				if (!(tmp & 0xC))
-					ret = PALRAM[0x00];
+					ret = READPAL(0x00);
 				else
-					ret = UPALRAM[((tmp & 0xC) >> 2) - 1];
+					ret = READUPAL(((tmp & 0xC) >> 2) - 1);
 			} else
-				ret = PALRAM[tmp & 0x1F];
-			if (GRAYSCALE)
-				ret &= 0x30;
+				ret = READPAL(tmp & 0x1F);
 			#ifdef FCEUDEF_DEBUGGER
 			if (!fceuindbg)
 			#endif
@@ -730,9 +803,14 @@ static DECLFR(A2007) {
 				if (PPU_hook) PPU_hook(tmp);
 				PPUGenLatch = VRAMBuffer;
 				if (tmp < 0x2000) {
+
 					if (debug_loggingCD)
 						LogAddress = GetCHRAddress(tmp);
-					VRAMBuffer = VPage[tmp >> 10][tmp];
+					if(MMC5Hack && newppu)
+						VRAMBuffer = *MMC5BGVRAMADR(tmp);
+					else
+						VRAMBuffer = VPage[tmp >> 10][tmp];
+
 				} else if (tmp < 0x3F00)
 					VRAMBuffer = vnapage[(tmp >> 10) & 0x3][tmp & 0x3FF];
 			}
@@ -785,6 +863,8 @@ static DECLFW(B2000) {
 
 static DECLFW(B2001) {
 	FCEUPPU_LineUpdate();
+	if (paldeemphswap)
+		V = (V&0x9F)|((V&0x40)>>1)|((V&0x20)<<1);
 	PPUGenLatch = V;
 	PPU[1] = V;
 	if (V & 0xE0)
@@ -880,6 +960,11 @@ static DECLFW(B2006) {
 static DECLFW(B2007) {
 	uint32 tmp = RefreshAddr & 0x3FFF;
 
+	if (debug_loggingCD) {
+		if(!cdloggerVideoDataSize && (tmp < 0x2000))
+			cdloggervdata[tmp] = 0;
+	}
+
 	if (newppu) {
 		PPUGenLatch = V;
 		RefreshAddr = ppur.get_2007access() & 0x3FFF;
@@ -892,8 +977,12 @@ static DECLFW(B2007) {
 			if (PPUCHRRAM & (1 << (tmp >> 10)))
 				VPage[tmp >> 10][tmp] = V;
 		} else if (tmp < 0x3F00) {
-			if (PPUNTARAM & (1 << ((tmp & 0xF00) >> 10)))
-				vnapage[((tmp & 0xF00) >> 10)][tmp & 0x3FF] = V;
+			if (QTAIHack && (qtaintramreg & 1)) {
+				QTAINTRAM[((((tmp & 0xF00) >> 10) >> ((qtaintramreg >> 1)) & 1) << 10) | (tmp & 0x3FF)] = V;
+			} else {
+				if (PPUNTARAM & (1 << ((tmp & 0xF00) >> 10)))
+					vnapage[((tmp & 0xF00) >> 10)][tmp & 0x3FF] = V;
+			}
 		} else {
 			if (!(tmp & 3)) {
 				if (!(tmp & 0xC))
@@ -918,6 +1007,7 @@ static DECLFW(B4014) {
 
 	for (x = 0; x < 256; x++)
 		X6502_DMW(0x2004, X6502_DMR(t + x));
+	SpriteDMA = V;
 }
 
 #define PAL(c)  ((c) + cc)
@@ -1010,7 +1100,7 @@ static void RefreshLine(int lastpixel) {
 	uint32 vofs;
 	int X1;
 
-	register uint8 *P = Pline;
+	uint8 *P = Pline;
 	int lasttile = lastpixel >> 3;
 	int numtiles;
 	static int norecurse = 0;	// Yeah, recursion would be bad.
@@ -1041,7 +1131,7 @@ static void RefreshLine(int lastpixel) {
 
 	if (!ScreenON && !SpriteON) {
 		uint32 tem;
-		tem = Pal[0] | (Pal[0] << 8) | (Pal[0] << 16) | (Pal[0] << 24);
+		tem = READPAL(0) | (READPAL(0) << 8) | (READPAL(0) << 16) | (READPAL(0) << 24);
 		tem |= 0x40404040;
 		FCEU_dwmemset(Pline, tem, numtiles * 8);
 		P += numtiles * 8;
@@ -1062,10 +1152,10 @@ static void RefreshLine(int lastpixel) {
 	}
 
 	//Priority bits, needed for sprite emulation.
-	Pal[0] |= 64;
-	Pal[4] |= 64;
-	Pal[8] |= 64;
-	Pal[0xC] |= 64;
+	PALRAM[0] |= 64;
+	PALRAM[4] |= 64;
+	PALRAM[8] |= 64;
+	PALRAM[0xC] |= 64;
 
 	//This high-level graphics MMC5 emulation code was written for MMC5 carts in "CL" mode.
 	//It's probably not totally correct for carts in "SL" mode.
@@ -1132,6 +1222,12 @@ static void RefreshLine(int lastpixel) {
 				#include "pputile.inc"
 			}
 			#undef PPU_BGFETCH
+		} if (QTAIHack) {
+			#define PPU_VRC5FETCH
+			for (X1 = firsttile; X1 < lasttile; X1++) {
+				#include "pputile.inc"
+			}
+			#undef PPU_VRC5FETCH
 		} else {
 			for (X1 = firsttile; X1 < lasttile; X1++) {
 				#include "pputile.inc"
@@ -1143,15 +1239,15 @@ static void RefreshLine(int lastpixel) {
 #undef RefreshAddr
 
 	//Reverse changes made before.
-	Pal[0] &= 63;
-	Pal[4] &= 63;
-	Pal[8] &= 63;
-	Pal[0xC] &= 63;
+	PALRAM[0] &= 63;
+	PALRAM[4] &= 63;
+	PALRAM[8] &= 63;
+	PALRAM[0xC] &= 63;
 
 	RefreshAddr = smorkus;
 	if (firsttile <= 2 && 2 < lasttile && !(PPU[1] & 2)) {
 		uint32 tem;
-		tem = Pal[0] | (Pal[0] << 8) | (Pal[0] << 16) | (Pal[0] << 24);
+		tem = READPAL(0) | (READPAL(0) << 8) | (READPAL(0) << 16) | (READPAL(0) << 24);
 		tem |= 0x40404040;
 		*(uint32*)Plinef = *(uint32*)(Plinef + 4) = tem;
 	}
@@ -1159,7 +1255,7 @@ static void RefreshLine(int lastpixel) {
 	if (!ScreenON) {
 		uint32 tem;
 		int tstart, tcount;
-		tem = Pal[0] | (Pal[0] << 8) | (Pal[0] << 16) | (Pal[0] << 24);
+		tem = READPAL(0) | (READPAL(0) << 8) | (READPAL(0) << 16) | (READPAL(0) << 24);
 		tem |= 0x40404040;
 
 		tcount = lasttile - firsttile;
@@ -1216,10 +1312,18 @@ static void Fixit1(void) {
 
 void MMC5_hb(int);		//Ugh ugh ugh.
 static void DoLine(void) {
-	int x;
-	uint8 *target = XBuf + (scanline << 8);
+	if (scanline >= 240 && scanline != totalscanlines) {
+		X6502_Run(256 + 69);
+		scanline++;
+		X6502_Run(16);
+		return;
+	}
 
-	if (MMC5Hack && (ScreenON || SpriteON)) MMC5_hb(scanline);
+	int x;
+	uint8 *target = XBuf + ((scanline < 240 ? scanline : 240) << 8);
+	u8* dtarget = XDBuf + ((scanline < 240 ? scanline : 240) << 8);
+
+	if (MMC5Hack) MMC5_hb(scanline);
 
 	X6502_Run(256);
 	EndRL();
@@ -1228,22 +1332,26 @@ static void DoLine(void) {
 		uint32 tem;
 		uint8 col;
 		if (gNoBGFillColor == 0xFF)
-			col = Pal[0];
+			col = READPAL(0);
 		else col = gNoBGFillColor;
 		tem = col | (col << 8) | (col << 16) | (col << 24);
-		tem |= 0x40404040;
+		tem |= 0x40404040; 
 		FCEU_dwmemset(target, tem, 256);
 	}
 
 	if (SpriteON)
 		CopySprites(target);
 
-	if (ScreenON || SpriteON) {	// Yes, very el-cheapo.
+	//greyscale handling (mask some bits off the color) ? ? ?
+	if (ScreenON || SpriteON)
+	{
 		if (PPU[1] & 0x01) {
 			for (x = 63; x >= 0; x--)
 				*(uint32*)&target[x << 2] = (*(uint32*)&target[x << 2]) & 0x30303030;
 		}
 	}
+
+	//some pathetic attempts at deemph
 	if ((PPU[1] >> 5) == 0x7) {
 		for (x = 63; x >= 0; x--)
 			*(uint32*)&target[x << 2] = ((*(uint32*)&target[x << 2]) & 0x3f3f3f3f) | 0xc0c0c0c0;
@@ -1254,6 +1362,10 @@ static void DoLine(void) {
 		for (x = 63; x >= 0; x--)
 			*(uint32*)&target[x << 2] = ((*(uint32*)&target[x << 2]) & 0x3f3f3f3f) | 0x80808080;
 
+	//write the actual deemph
+	for (x = 63; x >= 0; x--)
+		*(uint32*)&dtarget[x << 2] = ((PPU[1]>>5)<<0)|((PPU[1]>>5)<<8)|((PPU[1]>>5)<<16)|((PPU[1]>>5)<<24);
+
 	sphitx = 0x100;
 
 	if (ScreenON || SpriteON)
@@ -1357,10 +1469,10 @@ static void FetchSpriteData(void) {
 						C = VRAMADR(vadr);
 
 					if (SpriteON)
-						RENDER_LOG(vadr);
+						RENDER_LOGP(C);
 					dst.ca[0] = C[0];
 					if (SpriteON)
-						RENDER_LOG(vadr + 8);
+						RENDER_LOGP(C + 8);
 					dst.ca[1] = C[8];
 					dst.x = spr->x;
 					dst.atr = spr->atr;
@@ -1409,14 +1521,14 @@ static void FetchSpriteData(void) {
 					else
 						C = VRAMADR(vadr);
 					if (SpriteON)
-						RENDER_LOG(vadr);
+						RENDER_LOGP(C);
 					dst.ca[0] = C[0];
 					if (ns < 8) {
 						PPU_hook(0x2000);
 						PPU_hook(vadr);
 					}
 					if (SpriteON)
-						RENDER_LOG(vadr + 8);
+						RENDER_LOGP(C + 8);
 					dst.ca[1] = C[8];
 					dst.x = spr->x;
 					dst.atr = spr->atr;
@@ -1461,7 +1573,7 @@ static void RefreshSprites(void) {
 
 		int x = spr->x;
 		uint8 *C;
-		uint8 *VB;
+		int VB;
 
 		pixdata = ppulut1[spr->ca[0]] | ppulut2[spr->ca[1]];
 		J = spr->ca[0] | spr->ca[1];
@@ -1483,75 +1595,75 @@ static void RefreshSprites(void) {
 			}
 
 			C = sprlinebuf + x;
-			VB = (PALRAM + 0x10) + ((atr & 3) << 2);
+			VB = (0x10) + ((atr & 3) << 2);
 
 			if (atr & SP_BACK) {
 				if (atr & H_FLIP) {
-					if (J & 0x80) C[7] = VB[pixdata & 3] | 0x40;
+					if (J & 0x80) C[7] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x40) C[6] = VB[pixdata & 3] | 0x40;
+					if (J & 0x40) C[6] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x20) C[5] = VB[pixdata & 3] | 0x40;
+					if (J & 0x20) C[5] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x10) C[4] = VB[pixdata & 3] | 0x40;
+					if (J & 0x10) C[4] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x08) C[3] = VB[pixdata & 3] | 0x40;
+					if (J & 0x08) C[3] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x04) C[2] = VB[pixdata & 3] | 0x40;
+					if (J & 0x04) C[2] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x02) C[1] = VB[pixdata & 3] | 0x40;
+					if (J & 0x02) C[1] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x01) C[0] = VB[pixdata] | 0x40;
+					if (J & 0x01) C[0] = READPAL(VB | pixdata) | 0x40;
 				} else {
-					if (J & 0x80) C[0] = VB[pixdata & 3] | 0x40;
+					if (J & 0x80) C[0] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x40) C[1] = VB[pixdata & 3] | 0x40;
+					if (J & 0x40) C[1] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x20) C[2] = VB[pixdata & 3] | 0x40;
+					if (J & 0x20) C[2] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x10) C[3] = VB[pixdata & 3] | 0x40;
+					if (J & 0x10) C[3] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x08) C[4] = VB[pixdata & 3] | 0x40;
+					if (J & 0x08) C[4] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x04) C[5] = VB[pixdata & 3] | 0x40;
+					if (J & 0x04) C[5] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x02) C[6] = VB[pixdata & 3] | 0x40;
+					if (J & 0x02) C[6] = READPAL(VB | (pixdata & 3)) | 0x40;
 					pixdata >>= 4;
-					if (J & 0x01) C[7] = VB[pixdata] | 0x40;
+					if (J & 0x01) C[7] = READPAL(VB | pixdata) | 0x40;
 				}
 			} else {
 				if (atr & H_FLIP) {
-					if (J & 0x80) C[7] = VB[pixdata & 3];
+					if (J & 0x80) C[7] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x40) C[6] = VB[pixdata & 3];
+					if (J & 0x40) C[6] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x20) C[5] = VB[pixdata & 3];
+					if (J & 0x20) C[5] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x10) C[4] = VB[pixdata & 3];
+					if (J & 0x10) C[4] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x08) C[3] = VB[pixdata & 3];
+					if (J & 0x08) C[3] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x04) C[2] = VB[pixdata & 3];
+					if (J & 0x04) C[2] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x02) C[1] = VB[pixdata & 3];
+					if (J & 0x02) C[1] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x01) C[0] = VB[pixdata];
+					if (J & 0x01) C[0] = READPAL(VB | pixdata);
 				} else {
-					if (J & 0x80) C[0] = VB[pixdata & 3];
+					if (J & 0x80) C[0] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x40) C[1] = VB[pixdata & 3];
+					if (J & 0x40) C[1] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x20) C[2] = VB[pixdata & 3];
+					if (J & 0x20) C[2] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x10) C[3] = VB[pixdata & 3];
+					if (J & 0x10) C[3] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x08) C[4] = VB[pixdata & 3];
+					if (J & 0x08) C[4] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x04) C[5] = VB[pixdata & 3];
+					if (J & 0x04) C[5] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x02) C[6] = VB[pixdata & 3];
+					if (J & 0x02) C[6] = READPAL(VB | (pixdata & 3));
 					pixdata >>= 4;
-					if (J & 0x01) C[7] = VB[pixdata];
+					if (J & 0x01) C[7] = READPAL(VB | pixdata);
 				}
 			}
 		}
@@ -1561,7 +1673,6 @@ static void RefreshSprites(void) {
 }
 
 static void CopySprites(uint8 *target) {
-	uint8 n = ((PPU[1] & 4) ^ 4) << 1;
 	uint8 *P = target;
 
 	if (!spork) return;
@@ -1569,76 +1680,32 @@ static void CopySprites(uint8 *target) {
 
 	if (!rendersprites) return;	//User asked to not display sprites.
 
- loopskie:
+	if(!SpriteON) return;
+	
+	int start=8;
+	if(PPU[1] & 0x04)
+		start = 0;
+
+	for(int i=start;i<256;i++)
 	{
-		uint32 t = *(uint32*)(sprlinebuf + n);
-
-		if (t != 0x80808080) {
-			#ifdef LSB_FIRST
-			if (!(t & 0x80)) {
-				if (!(t & 0x40) || (P[n] & 0x40))		// Normal sprite || behind bg sprite
-					P[n] = sprlinebuf[n];
-			}
-
-			if (!(t & 0x8000)) {
-				if (!(t & 0x4000) || (P[n + 1] & 0x40))		// Normal sprite || behind bg sprite
-					P[n + 1] = (sprlinebuf + 1)[n];
-			}
-
-			if (!(t & 0x800000)) {
-				if (!(t & 0x400000) || (P[n + 2] & 0x40))	// Normal sprite || behind bg sprite
-					P[n + 2] = (sprlinebuf + 2)[n];
-			}
-
-			if (!(t & 0x80000000)) {
-				if (!(t & 0x40000000) || (P[n + 3] & 0x40))	// Normal sprite || behind bg sprite
-					P[n + 3] = (sprlinebuf + 3)[n];
-			}
-			#else
-			/* TODO:  Simplify */
-			if (!(t & 0x80000000)) {
-				if (!(t & 0x40000000))	// Normal sprite
-					P[n] = sprlinebuf[n];
-				else if (P[n] & 64)		// behind bg sprite
-					P[n] = sprlinebuf[n];
-			}
-
-			if (!(t & 0x800000)) {
-				if (!(t & 0x400000))	// Normal sprite
-					P[n + 1] = (sprlinebuf + 1)[n];
-				else if (P[n + 1] & 64)	// behind bg sprite
-					P[n + 1] = (sprlinebuf + 1)[n];
-			}
-
-			if (!(t & 0x8000)) {
-				if (!(t & 0x4000))		// Normal sprite
-					P[n + 2] = (sprlinebuf + 2)[n];
-				else if (P[n + 2] & 64)	// behind bg sprite
-					P[n + 2] = (sprlinebuf + 2)[n];
-			}
-
-			if (!(t & 0x80)) {
-				if (!(t & 0x40))		// Normal sprite
-					P[n + 3] = (sprlinebuf + 3)[n];
-				else if (P[n + 3] & 64)	// behind bg sprite
-					P[n + 3] = (sprlinebuf + 3)[n];
-			}
-			#endif
-		}
+		uint8 t = sprlinebuf[i];
+		if(!(t&0x80))
+			if (!(t & 0x40) || (P[i] & 0x40))		// Normal sprite || behind bg sprite
+				P[i] = t;
 	}
-	n += 4;
-	if (n) goto loopskie;
 }
 
 void FCEUPPU_SetVideoSystem(int w) {
 	if (w) {
-		scanlines_per_frame = 312;
+		scanlines_per_frame = dendy ? 262: 312;
 		FSettings.FirstSLine = FSettings.UsrFirstSLine[1];
 		FSettings.LastSLine = FSettings.UsrLastSLine[1];
+		//paldeemphswap = 1; // dendy has pal ppu, and pal ppu has these swapped
 	} else {
 		scanlines_per_frame = 262;
 		FSettings.FirstSLine = FSettings.UsrFirstSLine[0];
 		FSettings.LastSLine = FSettings.UsrLastSLine[0];
+		//paldeemphswap = 0;
 	}
 }
 
@@ -1661,17 +1728,23 @@ void FCEUPPU_Reset(void) {
 	kook = 0;
 	idleSynch = 1;
 
-	ppur.reset();
-	spr_read.reset();
+	new_ppu_reset = true; // delay reset of ppur/spr_read until it's ready to start a new frame
 }
 
 void FCEUPPU_Power(void) {
 	int x;
 
-	memset(NTARAM, 0x00, 0x800);
-	memset(PALRAM, 0x00, 0x20);
-	memset(UPALRAM, 0x00, 0x03);
-	memset(SPRAM, 0x00, 0x100);
+	// initialize PPU memory regions according to settings
+	FCEU_MemoryRand(NTARAM, 0x800, true);
+	FCEU_MemoryRand(PALRAM, 0x20, true);
+	FCEU_MemoryRand(SPRAM, 0x100, true);
+	// palettes can only store values up to $3F, and PALRAM X4/X8/XC are mirrors of X0 for rendering purposes (UPALRAM is used for $2007 readback)
+	for (x = 0; x < 0x20; ++x) PALRAM[x] &= 0x3F;
+	UPALRAM[0] = PALRAM[0x04];
+	UPALRAM[1] = PALRAM[0x08];
+	UPALRAM[2] = PALRAM[0x0C];
+	PALRAM[0x0C] = PALRAM[0x08] = PALRAM[0x04] = PALRAM[0x00];
+	PALRAM[0x1C] = PALRAM[0x18] = PALRAM[0x14] = PALRAM[0x10];
 	FCEUPPU_Reset();
 
 	for (x = 0x2000; x < 0x4000; x += 8) {
@@ -1724,6 +1797,13 @@ int FCEUPPU_Loop(int skip) {
 				TriggerNMI();
 		}
 		X6502_Run((scanlines_per_frame - 242) * (256 + 85) - 12);
+		if (overclock_enabled && vblankscanlines) {
+			if (!DMC_7bit || !skip_7bit_overclocking) {
+				overclocking = 1;
+				X6502_Run(vblankscanlines * (256 + 85) - 12);
+				overclocking = 0;
+			}
+		}
 		PPU_status &= 0x1f;
 		X6502_Run(256);
 
@@ -1754,7 +1834,7 @@ int FCEUPPU_Loop(int skip) {
 			kook ^= 1;
 		}
 		if (GameInfo->type == GIT_NSF)
-			X6502_Run((256 + 85) * 240);
+			X6502_Run((256 + 85) * normalscanlines);
 		#ifdef FRAMESKIP
 		else if (skip) {
 			int y;
@@ -1780,23 +1860,44 @@ int FCEUPPU_Loop(int skip) {
 		}
 		#endif
 		else {
-			int x, max, maxref;
-
 			deemp = PPU[1] >> 5;
-			for (scanline = 0; scanline < 240; ) {	//scanline is incremented in  DoLine.  Evil. :/
+
+			// manual samples can't play correctly with overclocking
+			if (DMC_7bit && skip_7bit_overclocking) // 7bit sample started before 240th line
+				totalscanlines = normalscanlines;
+			else
+				totalscanlines = normalscanlines + (overclock_enabled ? postrenderscanlines : 0);
+
+			for (scanline = 0; scanline < totalscanlines; ) {	//scanline is incremented in  DoLine.  Evil. :/
 				deempcnt[deemp]++;
-				DEBUG(FCEUD_UpdatePPUView(scanline, 1));
+
+				if (scanline < 240)
+					DEBUG(FCEUD_UpdatePPUView(scanline, 1));
+
 				DoLine();
+
+				if (scanline < normalscanlines || scanline == totalscanlines)
+					overclocking = 0;
+				else {
+					if (DMC_7bit && skip_7bit_overclocking) // 7bit sample started after 240th line
+						break;
+					overclocking = 1;
+				}
 			}
-			if (MMC5Hack && (ScreenON || SpriteON)) MMC5_hb(scanline);
-			for (x = 1, max = 0, maxref = 0; x < 7; x++) {
+			DMC_7bit = 0;
+
+			if (MMC5Hack) MMC5_hb(scanline);
+
+			//deemph nonsense, kept for complicated reasons (see SetNESDeemph_OldHacky implementation)
+			int maxref = 0;
+			for (int x = 1, max = 0; x < 7; x++) {
 				if (deempcnt[x] > max) {
 					max = deempcnt[x];
 					maxref = x;
 				}
 				deempcnt[x] = 0;
 			}
-			SetNESDeemph(maxref, 0);
+			SetNESDeemph_OldHacky(maxref, 0);
 		}
 	}	//else... to if(ppudead)
 
@@ -1807,7 +1908,6 @@ int FCEUPPU_Loop(int skip) {
 	} else
 	#endif
 	{
-		FCEU_PutImage();
 		return(1);
 	}
 }
@@ -1878,6 +1978,15 @@ void FCEUPPU_SaveState(void) {
 	RefreshAddrT = RefreshAddr;
 }
 
+uint32 FCEUPPU_PeekAddress()
+{
+	if (newppu)
+	{
+		return ppur.get_2007access() & 0x3FFF;
+	}
+
+	return RefreshAddr & 0x3FFF;
+}
 
 //---------------------
 int pputime = 0;
@@ -1887,21 +1996,34 @@ const int kFetchTime = 2;
 
 void runppu(int x) {
 	ppur.status.cycle = (ppur.status.cycle + x) % ppur.status.end_cycle;
-	X6502_Run(x);
+	if (!new_ppu_reset) // if resetting, suspend CPU until the first frame
+	{
+		X6502_Run(x);
+	}
 }
 
 //todo - consider making this a 3 or 4 slot fifo to keep from touching so much memory
 struct BGData {
 	struct Record {
-		uint8 nt, pecnt, at, pt[2];
+		uint8 nt, pecnt, at, pt[2], qtnt;
+		uint8 ppu1[8];
 
 		INLINE void Read() {
-			RefreshAddr = ppur.get_ntread();
+			NTRefreshAddr = RefreshAddr = ppur.get_ntread();
 			if (PEC586Hack)
 				ppur.s = (RefreshAddr & 0x200) >> 9;
+			else if (QTAIHack) {
+				qtnt = QTAINTRAM[((((RefreshAddr >> 10) & 3) >> ((qtaintramreg >> 1)) & 1) << 10) | (RefreshAddr & 0x3FF)];
+				ppur.s = qtnt & 0x3F;
+			}
 			pecnt = (RefreshAddr & 1) << 3;
 			nt = CALL_PPUREAD(RefreshAddr);
-			runppu(kFetchTime);
+			ppu1[0] = PPU[1];
+			runppu(1);
+			ppu1[1] = PPU[1];
+			runppu(1);
+
+
 
 			RefreshAddr = ppur.get_atread();
 			at = CALL_PPUREAD(RefreshAddr);
@@ -1913,33 +2035,57 @@ struct BGData {
 			at <<= 2;
 			//horizontal scroll clocked at cycle 3 and then
 			//vertical scroll at 251
+			ppu1[2] = PPU[1];
 			runppu(1);
 			if (PPUON) {
 				ppur.increment_hsc();
 				if (ppur.status.cycle == 251)
 					ppur.increment_vs();
 			}
+			ppu1[3] = PPU[1];
 			runppu(1);
 
 			ppur.par = nt;
 			RefreshAddr = ppur.get_ptread();
 			if (PEC586Hack) {
-				if (ScreenON)
-					RENDER_LOG(RefreshAddr | pecnt);
 				pt[0] = CALL_PPUREAD(RefreshAddr | pecnt);
-				runppu(kFetchTime);
+				ppu1[4] = PPU[1];
+				runppu(1);
+				ppu1[5] = PPU[1];
+				runppu(1);
 				pt[1] = CALL_PPUREAD(RefreshAddr | pecnt);
-				runppu(kFetchTime);
+				ppu1[6] = PPU[1];
+				runppu(1);
+				ppu1[7] = PPU[1];
+				runppu(1);
+			} else if (QTAIHack && (qtnt & 0x40)) {
+				pt[0] = *(CHRptr[0] + RefreshAddr);
+				ppu1[4] = PPU[1];
+				runppu(1);
+				ppu1[5] = PPU[1];
+				runppu(1);
+				RefreshAddr |= 8;
+				pt[1] = *(CHRptr[0] + RefreshAddr);
+				ppu1[6] = PPU[1];
+				runppu(1);
+				ppu1[7] = PPU[1];
+				runppu(1);
 			} else {
 				if (ScreenON)
 					RENDER_LOG(RefreshAddr);
 				pt[0] = CALL_PPUREAD(RefreshAddr);
-				runppu(kFetchTime);
+				ppu1[4] = PPU[1];
+				runppu(1);
+				ppu1[5] = PPU[1];
+				runppu(1);
 				RefreshAddr |= 8;
 				if (ScreenON)
 					RENDER_LOG(RefreshAddr);
 				pt[1] = CALL_PPUREAD(RefreshAddr);
-				runppu(kFetchTime);
+				ppu1[6] = PPU[1];
+				runppu(1);
+				ppu1[7] = PPU[1];
+				runppu(1);
 			}
 		}
 	};
@@ -1958,6 +2104,14 @@ static inline int PaletteAdjustPixel(int pixel) {
 
 int framectr = 0;
 int FCEUX_PPU_Loop(int skip) {
+
+	if (new_ppu_reset) // first frame since reset, time to initialize
+	{
+		ppur.reset();
+		spr_read.reset();
+		new_ppu_reset = false;
+	}
+
 	//262 scanlines
 	if (ppudead) {
 		// not quite emulating all the NES power up behavior
@@ -1988,12 +2142,20 @@ int FCEUX_PPU_Loop(int skip) {
 
 		ppur.status.sl = 241;	//for sprite reads
 
-		runppu(delay);			//X6502_Run(12);
+		//formerly: runppu(delay);
+		for(int dot=0;dot<delay;dot++)
+			runppu(1);
+
 		if (VBlankON) TriggerNMI();
-		if (PAL)
-			runppu(70 * (kLineTime) - delay);
-		else
-			runppu(20 * (kLineTime) - delay);
+		int sltodo = PAL?70:20;
+		
+		//formerly: runppu(20 * (kLineTime) - delay);
+		for(int S=0;S<sltodo;S++)
+		{
+			for(int dot=(S==0?delay:0);dot<kLineTime;dot++)
+				runppu(1);
+			ppur.status.sl++;
+		}
 
 		//this seems to run just before the dummy scanline begins
 		PPU_status = 0;
@@ -2016,22 +2178,30 @@ int FCEUX_PPU_Loop(int skip) {
 
 		//capture the initial xscroll
 		//int xscroll = ppur.fh;
-		//render 241 scanlines (including 1 dummy at beginning)
-		for (int sl = 0; sl < 241; sl++) {
+		//render 241/291 scanlines (1 dummy at beginning, dendy's 50 at the end)
+		//ignore overclocking!
+		for (int sl = 0; sl < normalscanlines; sl++) 
+		{
 			spr_read.start_scanline();
 
 			g_rasterpos = 0;
 			ppur.status.sl = sl;
 
+			linestartts = timestamp * 48 + X.count; // pixel timestamp for debugger
+
 			const int yp = sl - 1;
 			ppuphase = PPUPHASE_BG;
 
-			if (sl != 0) {
+			if (sl != 0 && sl < 241)  // ignore the invisible
+			{
 				DEBUG(FCEUD_UpdatePPUView(scanline = yp, 1));
 				DEBUG(FCEUD_UpdateNTView(scanline = yp, 1));
 			}
 
-			if (sl != 0) if (MMC5Hack && PPUON) MMC5_hb(yp);
+			//hack to fix SDF ship intro screen with split. is it right?
+			//well, if we didnt do this, we'd be passing in a negative scanline, so that's a sign something is fishy..
+			if(sl != 0)
+				if (MMC5Hack) MMC5_hb(yp);
 
 
 			//twiddle the oam buffers
@@ -2047,18 +2217,22 @@ int FCEUX_PPU_Loop(int skip) {
 			for (int xt = 0; xt < 32; xt++) {
 				bgdata.main[xt + 2].Read();
 
+				const uint8 blank = (gNoBGFillColor == 0xFF) ? READPAL(0) : gNoBGFillColor;
+
 				//ok, we're also going to draw here.
 				//unless we're on the first dummy scanline
-				if (sl != 0) {
+				if (sl != 0 && sl < 241) { // cape at 240 for dendy, its PPU does nothing afterwards
 					int xstart = xt << 3;
 					oamcount = oamcounts[renderslot];
 					uint8 * const target = XBuf + (yp << 8) + xstart;
+					uint8 * const dtarget = XDBuf + (yp << 8) + xstart;
 					uint8 *ptr = target;
+					uint8 *dptr = dtarget;
 					int rasterpos = xstart;
 
 					//check all the conditions that can cause things to render in these 8px
-					const bool renderspritenow = SpriteON && rendersprites && (xt > 0 || SpriteLeft8);
-					const bool renderbgnow = ScreenON && renderbg && (xt > 0 || BGLeft8);
+					const bool renderspritenow = SpriteON && (xt > 0 || SpriteLeft8);
+					const bool renderbgnow = ScreenON && (xt > 0 || BGLeft8);
 					for (int xp = 0; xp < 8; xp++, rasterpos++, g_rasterpos++) {
 						//bg pos is different from raster pos due to its offsetability.
 						//so adjust for that here
@@ -2066,14 +2240,28 @@ int FCEUX_PPU_Loop(int skip) {
 						const int bgpx = bgpos & 7;
 						const int bgtile = bgpos >> 3;
 
-						uint8 pixel = 0, pixelcolor;
+						uint8 pixel = 0;
+						uint8 pixelcolor = blank;
+
+						//according to qeed's doc, use palette 0 or $2006's value if it is & 0x3Fxx
+						if (!ScreenON && !SpriteON)
+						{
+							// if there's anything wrong with how we're doing this, someone please chime in
+							int addr = ppur.get_2007access();
+							if ((addr & 0x3F00) == 0x3F00)
+							{
+								pixel = addr & 0x1F;
+							}
+							pixelcolor = READPAL_MOTHEROFALL(pixel);
+						}
 
 						//generate the BG data
 						if (renderbgnow) {
 							uint8* pt = bgdata.main[bgtile].pt;
 							pixel = ((pt[0] >> (7 - bgpx)) & 1) | (((pt[1] >> (7 - bgpx)) & 1) << 1) | bgdata.main[bgtile].at;
 						}
-						pixelcolor = PALRAM[pixel];
+						if (renderbg)
+							pixelcolor = READPALNOGS(pixel);
 
 						//look for a sprite to be drawn
 						bool havepixel = false;
@@ -2116,11 +2304,27 @@ int FCEUX_PPU_Loop(int skip) {
 
 								//bring in the palette bits and palettize
 								spixel |= (oam[2] & 3) << 2;
-								pixelcolor = PALRAM[0x10 + spixel];
+
+								if (rendersprites)
+									pixelcolor = READPALNOGS(0x10 + spixel);
 							}
 						}
 
-						*ptr++ = PaletteAdjustPixel(pixelcolor);
+						//apply grayscale.. kind of clunky
+						//really we need to read the entire palette instead of just ppu1
+						//this will be needed for special color effects probably (very fine rainbows and whatnot?)
+						//are you allowed to chang the palette mid-line anyway? well you can definitely change the grayscale flag as we know from the FF1 "polygon" effect
+						if(bgdata.main[xt+2].ppu1[xp]&1)
+							pixelcolor &= 0x30;
+
+						//this does deemph stuff inside it.. which is probably wrong...
+						*ptr = PaletteAdjustPixel(pixelcolor);
+
+						ptr++;
+
+						//grab deemph..
+						//I guess this works the same way as the grayscale, ideally?
+						*dptr++ = bgdata.main[xt+2].ppu1[xp]>>5;
 					}
 				}
 			}
@@ -2220,7 +2424,7 @@ int FCEUX_PPU_Loop(int skip) {
 						runppu(1);
 						garbage_todo = 0;
 					}
-					if ((sl != 0) && ppur.status.cycle == 256)
+					if ((sl != 0 && sl < 241) && ppur.status.cycle == 256)
 					{
 						runppu(1);
 						//at 257: 3d world runner is ugly if we do this at 256
@@ -2247,6 +2451,14 @@ int FCEUX_PPU_Loop(int skip) {
 					}
 				}
 
+				//blind attempt to replicate old ppu functionality
+				if(s == 2 && PPUON)
+				{
+					if (GameHBIRQHook2) {
+						GameHBIRQHook2();
+					}
+				}
+
 				if (realSprite) runppu(kFetchTime);
 
 
@@ -2301,7 +2513,9 @@ int FCEUX_PPU_Loop(int skip) {
 				runppu(1);
 		}	//scanline loop
 
-		if (MMC5Hack && PPUON) MMC5_hb(240);
+		DMC_7bit = 0;
+
+		if (MMC5Hack) MMC5_hb(240);
 
 		//idle for one line
 		runppu(kLineTime);
@@ -2309,7 +2523,5 @@ int FCEUX_PPU_Loop(int skip) {
 	}
 
 finish:
-	FCEU_PutImage();
-
 	return 0;
 }
